/*
	Copyright (C) 2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <mtcelledit.h>



static int	exit_val	= 0,
		ofile_type	= CED_FILE_TYPE_TSV_CONTENT,
		ofile_rows	= 1000,
		ofile_cols	= 1000,
		ofile_tot	= 1000000
		;



static int print_version (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	printf ( "random - Part of %s\n", VERSION );
	printf ( "Creates random sheet data and saves it.\n\n" );

	return 1;			// Stop parsing
}

static int print_help (
	mtArg		* mtarg,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	print_version ( mtarg, arg, argc, argv );

	printf (
	"Usage: random [ [OPTION] [FILE] ]...\n"
	"\n"
	"Options:\n"
	"  --help               Output this help.\n"
	"  --version            Output version information.\n"
	"  -tsv                 Output file = TSV.\n"
	"  -csv                 Output file = CSV.\n"
	"  -ledger              Output file = LEDGER.\n"
	"  -rows INTEGER        Output file = INTEGER rows (default=1000).\n"
	"  -cols INTEGER        Output file = INTEGER cols (default=1000).\n"
	"  -tot INTEGER         Random iterations (default=1000000).\n"
	"\n"
	"No file is ever overwritten or deleted.\n"
	"\n"
		);

	return 1;			// Stop parsing
}



#define BUF_SIZE	8



static int file_func (
	char		* filename
	)
{
	CedSheet	* sheet = NULL;
	int		i,
			j,
			r,
			c
			;
	double		dnum;
	char		txt [ BUF_SIZE ];


	if ( 0 == mtkit_file_valid ( filename ) )
	{
		fprintf ( stderr, "File already exists: %s\n", filename );

		goto error;
	}

	sheet = ced_sheet_new ();

	if ( ! sheet )
	{
		fprintf ( stderr, "Unable to create new sheet.\n" );

		goto error;
	}

	txt[BUF_SIZE - 1] = 0;

	for ( i = 0; i < ofile_tot; i++ )
	{
		r = 1 + rand () % ofile_rows;
		c = 1 + rand () % ofile_cols;

		dnum = rand () % 1000000;
		dnum = dnum / 1000;

		ced_sheet_set_cell_value ( sheet, r, c, dnum );

		r = 1 + rand () % ofile_rows;
		c = 1 + rand () % ofile_cols;

		for ( j = 0; j < (BUF_SIZE - 1); j++ )
		{
			txt[j] = 32 + rand () % 95;
		}

		ced_sheet_set_cell_text ( sheet, r, c, txt );
	}

	if ( ced_sheet_save ( sheet, filename, ofile_type ) )
	{
		fprintf ( stderr, "Unable to save file %s type %i.\n",
			filename, ofile_type );

		goto error;
	}

	ced_sheet_destroy ( sheet );

	return 0;

error:
	ced_sheet_destroy ( sheet );

	exit_val = 1;

	return 1;
}

static int error_func (
	int		error,
	int		arg,
	int		argc,
	char		* argv[]
	)
{
	fprintf ( stderr, "error_func: Argument ERROR! - num = %i arg = %i/%i",
		error, arg, argc );

	if ( arg < argc )
	{
		fprintf ( stderr, " '%s'", argv[arg] );
	}

	fprintf ( stderr, "\n" );

	return 0;			// Keep parsing
}

int main (
	int		argc,
	char		* argv[]
	)
{
	int		tmp;

	mtArg		arg_list[] = {
	{ "-help",	MTKIT_ARG_SWITCH, &tmp, 0, print_help },
	{ "-version",	MTKIT_ARG_SWITCH, &tmp, 0, print_version },
	{ "rows",	MTKIT_ARG_INT,	&ofile_rows },
	{ "cols",	MTKIT_ARG_INT,	&ofile_cols },
	{ "tot",	MTKIT_ARG_INT,	&ofile_tot },
	{ "tsv",	MTKIT_ARG_SWITCH, &ofile_type, CED_FILE_TYPE_TSV_VALUE },
	{ "csv",	MTKIT_ARG_SWITCH, &ofile_type, CED_FILE_TYPE_CSV_VALUE },
	{ "ledger",	MTKIT_ARG_SWITCH, &ofile_type, CED_FILE_TYPE_LEDGER },
	{ NULL }
	};


	if ( ced_init () )
	{
		fprintf ( stderr, "Unable to start libmtcelledit.\n\n" );

		exit_val = 1;
	}
	else
	{
		mtkit_arg_parse ( argc, argv, arg_list, file_func, error_func );
	}

	return exit_val;
}
