/*
	Copyright (C) 2008-2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "be.h"



static CedSheet		* pref_paste_sheet;
static int		pref_r1,
			pref_c1,
			pref_r2,
			pref_c2;
static unsigned int	old_cursor_r1,
			old_cursor_c1,
			old_cursor_r2,
			old_cursor_c2;


static void be_cellpref_cleanup_sheet ()
{
	ced_sheet_destroy ( pref_paste_sheet );
	pref_paste_sheet = NULL;
}

void be_cellpref_cleanup ( CedSheet * sheet )
{
	if ( sheet )
	{
		// Restore the cursor to its original position

		sheet->prefs.cursor_r1 = old_cursor_r1;
		sheet->prefs.cursor_r2 = old_cursor_r2;
		sheet->prefs.cursor_c1 = old_cursor_c1;
		sheet->prefs.cursor_c2 = old_cursor_c2;
	}

	be_cellpref_cleanup_sheet ();
}

int be_prepare_prefs_set ( CedSheet * sheet )
{
	if ( ! sheet )
	{
		return 1;
	}

	if ( pref_paste_sheet )
	{
		be_cellpref_cleanup_sheet ();
	}

	cedui_cellprefs_init ( sheet, &pref_r1, &pref_c1, &pref_r2, &pref_c2,
		&pref_paste_sheet );
	if ( ! pref_paste_sheet )
	{
		return 1;
	}

	// NOTE: we have to do this as well as pref_*
	old_cursor_r1 = sheet->prefs.cursor_r1;
	old_cursor_r2 = sheet->prefs.cursor_r2;
	old_cursor_c1 = sheet->prefs.cursor_c1;
	old_cursor_c2 = sheet->prefs.cursor_c2;

	return 0;			// Success
}

int be_cellpref_bulk_import (
	mtPrefs		* prefs,
	CedCellPrefs	* cell_prefs
	)
{
	mtBulkInt	b_int[] = {
		{ CED_FILE_PREFS_CELL_ALIGN_HORIZONTAL,		&cell_prefs->align_horizontal },
		{ CED_FILE_PREFS_CELL_COLOR_BACKGROUND,		&cell_prefs->color_background },
		{ CED_FILE_PREFS_CELL_COLOR_FOREGROUND,		&cell_prefs->color_foreground },
		{ CED_FILE_PREFS_CELL_FORMAT,			&cell_prefs->format },
		{ CED_FILE_PREFS_CELL_NUM_DECIMAL_PLACES,	&cell_prefs->num_decimal_places },
		{ CED_FILE_PREFS_CELL_NUM_ZEROS,		&cell_prefs->num_zeros },
		{ CED_FILE_PREFS_CELL_BOLD_TEXT,		&cell_prefs->bold_text },
		{ CED_FILE_PREFS_CELL_LOCKED,			&cell_prefs->locked },
		{ CED_FILE_PREFS_CELL_BORDER_TYPE,		&cell_prefs->border_type },
		{ CED_FILE_PREFS_CELL_BORDER_COLOR,		&cell_prefs->border_color },
		{ NULL }
		};
	mtBulkStr	b_str[] = {
		{ CED_FILE_PREFS_CELL_FORMAT_DATETIME,		&cell_prefs->format_datetime },
		{ CED_FILE_PREFS_CELL_NUM_THOUSANDS,		&cell_prefs->num_thousands },
		{ CED_FILE_PREFS_CELL_TEXT_PREFIX,		&cell_prefs->text_prefix },
		{ CED_FILE_PREFS_CELL_TEXT_SUFFIX,		&cell_prefs->text_suffix },
		{ NULL }
		};

	if ( ! prefs || ! cell_prefs )
	{
		return 1;
	}

	mtkit_prefs_block_callback ( prefs );
	mtkit_prefs_bulk_set ( prefs, b_int, NULL, b_str );
	mtkit_prefs_unblock_callback ( prefs );

	return 0;
}

void be_cellpref_changed (
	mtPrefItemValue	* piv,
	int		callback_data,
	void		* callback_ptr
	)
{
	int		num = 0;
	char		* charp = NULL;


	switch ( callback_data )
	{
	case CEDUI_CELLPREFS_align_horizontal:
	case CEDUI_CELLPREFS_color_background:
	case CEDUI_CELLPREFS_color_foreground:
	case CEDUI_CELLPREFS_format:
	case CEDUI_CELLPREFS_num_decimal_places:
	case CEDUI_CELLPREFS_num_zeros:
	case CEDUI_CELLPREFS_bold_text:
	case CEDUI_CELLPREFS_locked:
	case CEDUI_CELLPREFS_border_type:
	case CEDUI_CELLPREFS_border_color:
		mtkit_strtoi ( piv->value, &num, NULL, 0 );
		break;

	case CEDUI_CELLPREFS_format_datetime:
	case CEDUI_CELLPREFS_num_thousands:
	case CEDUI_CELLPREFS_text_prefix:
	case CEDUI_CELLPREFS_text_suffix:
		charp = piv->value;
		break;

	default:
		return;
	}

	fe_commit_prefs_set ( callback_data, num, charp, 1, callback_ptr );
}

mtPrefs * be_cellpref_init (
	CedSheet	* sheet,
	mtPrefCB	callback,
	void		* ptr
	)
{
	mtPrefs		* prefs;
	CedCell		* cell = NULL;
	CedCellPrefs	* cell_prefs;
	mtPrefItem item_table[] = {
	{ CED_FILE_PREFS_CELL_ALIGN_HORIZONTAL, MTKIT_PREF_TYPE_OPTION, "0", _("Horizontal text alignment"), callback, CEDUI_CELLPREFS_align_horizontal, "None	Left	Centre	Right", ptr },
	{ CED_FILE_PREFS_CELL_COLOR_BACKGROUND, MTKIT_PREF_TYPE_RGB, "16777215", _("Cell background colour"), callback, CEDUI_CELLPREFS_color_background, NULL, ptr },
	{ CED_FILE_PREFS_CELL_COLOR_FOREGROUND, MTKIT_PREF_TYPE_RGB, "0", _("Text foreground colour"), callback, CEDUI_CELLPREFS_color_foreground, NULL, ptr },
	{ CED_FILE_PREFS_CELL_FORMAT, MTKIT_PREF_TYPE_OPTION, "0", _("Cell format"), callback, CEDUI_CELLPREFS_format, "General	Text	Fixed Decimal	Hexadecimal	Binary	Scientific	Percentage	Datetime", ptr },
	{ CED_FILE_PREFS_CELL_FORMAT_DATETIME, MTKIT_PREF_TYPE_STR, "", _("Datetime format e.g. d/m/y HH:MM:SS"), callback, CEDUI_CELLPREFS_format_datetime, "32", ptr },
	{ CED_FILE_PREFS_CELL_NUM_DECIMAL_PLACES, MTKIT_PREF_TYPE_INT, "0", _("Number of decimal places for fixed type"), callback, CEDUI_CELLPREFS_num_decimal_places, "0	99", ptr },
	{ CED_FILE_PREFS_CELL_NUM_THOUSANDS, MTKIT_PREF_TYPE_STR, "", _("Thousands separator character"), callback, CEDUI_CELLPREFS_num_thousands, "1", ptr },
	{ CED_FILE_PREFS_CELL_NUM_ZEROS, MTKIT_PREF_TYPE_INT, "0", _("Leading zeros"), callback, CEDUI_CELLPREFS_num_zeros, "0	32", ptr },
	{ CED_FILE_PREFS_CELL_BOLD_TEXT, MTKIT_PREF_TYPE_BOOL, "0", _("Bold"), callback, CEDUI_CELLPREFS_bold_text, NULL, ptr },
	{ CED_FILE_PREFS_CELL_LOCKED, MTKIT_PREF_TYPE_BOOL, "0", _("Cell locked - read only"), callback, CEDUI_CELLPREFS_locked, NULL, ptr },
	{ CED_FILE_PREFS_CELL_BORDER_TYPE, MTKIT_PREF_TYPE_INT, "0", _("Border type"), callback, CEDUI_CELLPREFS_border_type, NULL, ptr },
	{ CED_FILE_PREFS_CELL_BORDER_COLOR, MTKIT_PREF_TYPE_RGB, "0", _("Border colour"), callback, CEDUI_CELLPREFS_border_color, NULL, ptr },
	{ CED_FILE_PREFS_CELL_TEXT_PREFIX, MTKIT_PREF_TYPE_STR, "", _("Cell text prefix"), callback, CEDUI_CELLPREFS_text_prefix, NULL, ptr },
	{ CED_FILE_PREFS_CELL_TEXT_SUFFIX, MTKIT_PREF_TYPE_STR, "", _("Cell text suffix"), callback, CEDUI_CELLPREFS_text_suffix, NULL, ptr },
	{ NULL }
	};


	if ( be_prepare_prefs_set ( sheet ) )
	{
		return NULL;
	}

	cell = ced_sheet_get_cell ( sheet, old_cursor_r1, old_cursor_c1 );

	if ( cell && cell->prefs )
	{
		cell_prefs = cell->prefs;
	}
	else
	{
		cell_prefs = ced_cell_prefs_default ();
	}

	// Load window prefs from disk, cell prefs from sheet/default
	prefs = fe_load_pref_window_prefs ( item_table );
	if ( prefs && cell_prefs )
	{
		be_cellpref_bulk_import ( prefs, cell_prefs );
	}

	return prefs;
}

void be_save_pref_window_prefs (
	mtPrefs		* prefs
	)
{
	mtPrefTrans
		winprefs_main_main[] = {
		{ CED_INIFILE_PREFS_WINDOW"_x", CED_INIFILE_PREFS_WINDOW"_x" },
		{ CED_INIFILE_PREFS_WINDOW"_y", CED_INIFILE_PREFS_WINDOW"_y" },
		{ CED_INIFILE_PREFS_WINDOW"_w", CED_INIFILE_PREFS_WINDOW"_w" },
		{ CED_INIFILE_PREFS_WINDOW"_h", CED_INIFILE_PREFS_WINDOW"_h" },
		{ NULL }
		},
		winprefs_shared_main[] = {
		{ CED_INIFILE_SHARED_WINDOW"_x", CED_INIFILE_PREFS_WINDOW"_x" },
		{ CED_INIFILE_SHARED_WINDOW"_y", CED_INIFILE_PREFS_WINDOW"_y" },
		{ CED_INIFILE_SHARED_WINDOW"_w", CED_INIFILE_PREFS_WINDOW"_w" },
		{ CED_INIFILE_SHARED_WINDOW"_h", CED_INIFILE_PREFS_WINDOW"_h" },
		{ NULL }
		};


	mtkit_prefs_value_copy ( prefs_file (), prefs, winprefs_shared_main );
	mtkit_prefs_value_copy ( prefs, prefs_file (), winprefs_main_main );
}

void be_load_pref_window_prefs (
	mtPrefs		* prefs
	)
{
	mtPrefTrans
		winprefs_main_shared[] = {
		{ CED_INIFILE_PREFS_WINDOW"_x", CED_INIFILE_SHARED_WINDOW"_x" },
		{ CED_INIFILE_PREFS_WINDOW"_y", CED_INIFILE_SHARED_WINDOW"_y" },
		{ CED_INIFILE_PREFS_WINDOW"_w", CED_INIFILE_SHARED_WINDOW"_w" },
		{ CED_INIFILE_PREFS_WINDOW"_h", CED_INIFILE_SHARED_WINDOW"_h" },
		{ NULL }
		};


	mtkit_prefs_value_copy ( prefs, prefs_file (), winprefs_main_shared );
}

int be_commit_prefs_set (
	CedSheet	* sheet,
	CuBook		* cubook,
	int		pref_id,
	int		pref_num,
	char		* pref_charp
	)
{
	return cedui_cellprefs_change ( cubook, sheet,
		pref_r1, pref_c1, pref_r2, pref_c2,
		pref_paste_sheet, pref_id, pref_charp, pref_num );
}

int be_commit_prefs_set_check (
	int		res,
	CedSheet	* sheet,
	int		change_cursor,
	int		pref_id
	)
{
	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return 1;
	}

	if (	change_cursor &&
		(	pref_id == CEDUI_CELLPREFS_color_background ||
			pref_id == CEDUI_CELLPREFS_color_foreground
			)
		)
	{
/*
Set the cursor to invisible, i.e. 0,0 -> 0,0 so that the user can see
background colour updates
*/
		sheet->prefs.cursor_r1 = 0;
		sheet->prefs.cursor_r2 = 0;
		sheet->prefs.cursor_c1 = 0;
		sheet->prefs.cursor_c2 = 0;
	}

	return 0;
}

