/*
	Copyright (C) 2013-2014 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "tk_qt4.h"



void MainWindow :: pressOptionsGraph ()
{
	if ( graphTextEdit->hasFocus () )
	{
		tabWidget->hide ();
		viewMain->setFocus ();
	}
	else
	{
		tabWidget->show ();
		tabWidget->setCurrentIndex ( TAB_GRAPH );
		graphTextEdit->setFocus ( Qt::OtherFocusReason );
	}
}

void MainWindow :: projectGraphStoreChanges ()
{
	char		* txt = NULL;
	CedBookFile	* bookfile;


	// Checks for changes, if so save them to the current graph
	// (active_graph).

	if ( ! graphTextEdit->document ()->isModified () )
	{
		// No changes so nothing to store

		return;
	}

	bookfile = cuf_file_get_graph ( cedFile );
	if ( ! bookfile )
	{
		return;
	}

	QByteArray tmp = graphTextEdit->toPlainText ().toUtf8 ();
	txt = tmp.data ();

	if ( txt )
	{
		txt = strdup ( txt );
	}

	// Free old memory and use newly allocated text
	free ( bookfile->mem );
	bookfile->mem = txt;

	if ( txt )
	{
		bookfile->size = strlen ( txt );
	}

	ced_book_timestamp_file ( bookfile );

	updateChangesChores ( 0, 1 );

	graphTextEdit->document ()->setModified ( false );

	projectGraphRedraw ();		// Draw the new graph
}

void MainWindow :: graphChanged ( int i )
{
	QString		text;
	char		* graph_text = NULL;
	int		graph_text_len = 0;
	CedBookFile	* bookfile;
	CedBook		* book;


	projectGraphStoreChanges ();

	text = buttonGraph->text ();
	book = cedFile->cubook->book;

	if ( text.isEmpty () )
	{
		mtkit_strfreeswap ( &book->prefs.active_graph, NULL );
	}
	else
	{
		bookfile = ced_graph_get ( book, text.toUtf8 ().data () );
		if ( bookfile && bookfile->mem )
		{
			graph_text = bookfile->mem;
			graph_text_len = bookfile->size;
		}

		mtkit_strfreeswap ( &book->prefs.active_graph,
			text.toUtf8 ().data () );
	}

	graphTextEdit->setPlainText ( mtQEX::qstringFromC ( graph_text,
		graph_text_len ) );

	graphTextEdit->document ()->setModified ( false );

	projectGraphRedraw ();		// Draw the new graph
}

void MainWindow :: pressGraphNew ()
{
	char		* newname;


	projectGraphStoreChanges ();

	newname = be_graph_new ( cedFile->cubook->book );

	if ( newname )
	{
		updateGraph ( newname );
	}
	else
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			tr ("Unable to create a new graph.") );
	}

	updateChangesChores ( 0, 1 );
}

void MainWindow :: pressGraphDuplicate ()
{
	char		* newname = NULL;


	projectGraphStoreChanges ();

	newname = be_graph_duplicate ( cedFile->cubook );

	if ( newname )
	{
		updateGraph ( newname );
		free ( newname );
	}
	else
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			tr ("Unable to duplicate the current graph.") );
	}

	updateChangesChores ( 0, 1 );
}

int MainWindow :: projectRenameGraph ( QString newName )
{
	char		* old_name,
			* new_name;
	CedBookFile	* old;
	CedBook		* book = cedFile->cubook->book;
	QByteArray	tmp = newName.toUtf8 ();


	new_name = tmp.data ();

	if ( ! new_name || new_name[0] == 0 )
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			tr ("Bad graph name.") );

		return 1;
	}
	else if ( ced_graph_get ( book, new_name ) )
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			tr ("Graph name already exists.") );

		return 1;
	}

	old_name = cedFile->cubook->book->prefs.active_graph;
	old = ced_graph_get ( book, old_name );

	if (	! old ||
		! ced_graph_new ( book, old->mem, old->size, new_name )
		)
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			tr ("Unable to rename graph.") );

		return 1;
	}
	else
	{
		old->mem = NULL;
		ced_graph_destroy ( book, old_name );
	}

	updateGraph ( new_name );
	updateChangesChores ( 0, 1 );

	return 0;			// Success
}

void MainWindow :: pressGraphRename ()
{
	char		* old_name;


	old_name = cedFile->cubook->book->prefs.active_graph;

	if ( ! old_name )
	{
		return;
	}

	projectGraphStoreChanges ();

	for ( int res = 1; res; )
	{
		bool	ok;
		QString new_name = QInputDialog::getText ( this,
			tr ( "Rename Graph" ),
			tr ( "New Graph Name:" ),
			QLineEdit::Normal,
			mtQEX::qstringFromC ( old_name ),
			&ok );


		if ( ok )
		{
			res = projectRenameGraph ( new_name );
		}
		else
		{
			break;
		}
	}
}

void MainWindow :: pressGraphDelete ()
{
	int		res,
			gnum;
	CedBook		* book;


	res = QMessageBox :: question ( this, tr ( "Question" ),
		tr ("Do you really want to delete this graph?"),
		QMessageBox::Yes | QMessageBox::No,
		QMessageBox::No );

	if ( res != QMessageBox::Yes )
	{
		return;
	}

	gnum = buttonGraph->currentIndex ();

	book = cedFile->cubook->book;
	if ( ced_graph_destroy ( book, book->prefs.active_graph ) )
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			tr ("Unable to delete this graph.") );

		return;
	}

	updateGraph ( NULL );

	if ( gnum > 0 )
	{
		gnum --;
	}

	buttonGraph->setCurrentIndex ( gnum );

	updateChangesChores ( 0, 1 );
}

void MainWindow :: projectGraphRedraw ()
{
	int		err	= -1;
	mtImage		* im;
	double		scale;
	CedBook		* book;


	scale = prefs_get_double ( CED_INIFILE_GRAPH_SCALE );
	book = cedFile->cubook->book;
	im = ced_graph_render_mtimage ( book, book->prefs.active_graph, &err,
		scale );

	graphWidget->setImage ( im );

	if ( err >= 0 )
	{
		QTextCursor	c = graphTextEdit->textCursor ();


		c.movePosition ( QTextCursor::End );
		c.setPosition ( err, QTextCursor::KeepAnchor );

		graphTextEdit->setTextCursor ( c );
	}
}

void MainWindow :: pressGraphRedraw ()
{
	projectGraphStoreChanges ();
	projectGraphRedraw ();
}

void MainWindow :: pressGraphExport ()
{
	char		* graphname;


	graphname = cedFile->cubook->book->prefs.active_graph;

	if ( ! graphname )
	{
		return;
	}

	projectGraphStoreChanges ();

	QStringList	list;
	QString		last = mtQEX::qstringFromC ( prefs_get_string (
				CED_INIFILE_LAST_DIR ) ) +
				MTKIT_DIR_SEP + "export";


	// Order as per CED_GRAPH_TYPE_*
	list	<< "EPS"
		<< "PDF"
		<< "PNG"
		<< "PS"
		<< "SVG"
		;

	while ( 1 )
	{
		MySaveFileDialog dialog ( this, tr ( "Export Graph" ),
			list, lastExportGraphType, last.toUtf8 ().data () );

		if ( ! dialog.exec () )
		{
			break;
		}

		QStringList	fileList = dialog.selectedFiles ();
		QString		filename = fileList.at ( 0 );


		if ( ! filename.isEmpty () )
		{
			lastExportGraphType = dialog.getFormat ();

			if ( ced_graph_render_file ( cedFile->cubook->book,
				graphname, filename.toUtf8 ().data (),
				lastExportGraphType, NULL,
				prefs_get_double ( CED_INIFILE_GRAPH_SCALE ) )
				)
			{
				QMessageBox :: critical ( this, tr ( "Error" ),
					tr ("Unable to export graph.") );

				continue;
			}

			be_remember_last_dir ( filename.toUtf8 ().data () );
		}

		break;
	}
}

void MainWindow :: pressGraphSClipboard ()
{
	char		txt [ 2000 ] = { 0 };


	if ( be_graph_selection_clip ( projectGetSheet (), txt, sizeof ( txt ) )
		)
	{
		sprintf ( txt, _("No sheet available") );
	}

	QClipboard	* c = QApplication::clipboard ();

	if ( c )
	{
		c->setText ( mtQEX::qstringFromC ( txt ) );
	}
}

