/*
	Copyright (C) 2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "tk_qt4.h"



int MainWindow :: projectSetFont (
	char		* filename,
	int		size
	)
{
	mtFont		* newfont;


	newfont = mtpixel_font_new ( filename, filename ? 0 : size );
	if ( ! newfont )
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			QString ( tr ( "Unable to open font file %1 size %2." )
			).arg ( filename ).arg ( size ) );

		return 1;
	}

	if ( filename )
	{
		mtpixel_font_set_size ( newfont, size );
	}

	mtpixel_font_destroy ( render.font );
	render.font = newfont;

	updateViewConfig ();

	return 0;
}

mtFont * MainWindow :: projectGetFont ()
{
	return render.font;
}

void pref_change_font (
	mtPrefItemValue	* piv,
	int		callback_data,
	void		* callback_ptr
	)
{
	MainWindow	* mw		= (MainWindow *)callback_ptr;
	char		* filename	= NULL;
	int		size		= 16;


	switch ( prefs_get_int ( CED_INIFILE_FONT_TYPE ) )
	{
	case 0:
		size = 8;
		break;

	case 2:
		filename = prefs_get_string ( CED_INIFILE_FONT_FILENAME );
		size = prefs_get_int ( CED_INIFILE_FONT_SIZE );
		break;
	}

	if ( mw->projectSetFont ( filename, size ) )
	{
		mw->projectSetFont ( NULL, 16 );
	}
}

void pref_change_row_pad (
	mtPrefItemValue	* piv,
	int		callback_data,
	void		* callback_ptr
	)
{
	MainWindow	* mw = (MainWindow *)callback_ptr;


	mw->updateViewConfig ();
}

void pref_change_graph_scale (
	mtPrefItemValue	* piv,
	int		callback_data,
	void		* callback_ptr
	)
{
	MainWindow	* mw = (MainWindow *)callback_ptr;


	mw->pressGraphRedraw ();
}

void pref_change_recent_filename_len (
	mtPrefItemValue	* piv,
	int		callback_data,
	void		* callback_ptr
	)
{
	MainWindow	* mw = (MainWindow *)callback_ptr;


	mw->updateRecentFiles ();
}

void fe_save_pref_window_prefs (
	mtPrefs		* prefs
	)
{
	be_save_pref_window_prefs ( prefs );
	mtQEX :: prefsWindowMirrorPrefs ( prefs_file (), prefs );
}

mtPrefs * fe_load_pref_window_prefs (
	mtPrefItem	* table
	)
{
	mtPrefs		* prefs;


	prefs = mtkit_prefs_new ( table );
	mtQEX :: prefsInitPrefs ( prefs );

	if ( mtQEX :: prefsWindowMirrorPrefs ( prefs, prefs_file () ) )
	{
		mtkit_prefs_destroy ( prefs );

		return NULL;
	}

	be_load_pref_window_prefs ( prefs );

	return prefs;
}

void fe_commit_prefs_set (
	int		pref_id,
	int		pref_num,
	char		* pref_charp,
	int		change_cursor,
	void		* callback_ptr
	)
{
	CedSheet	* sheet = mainwindow->projectGetSheet ();
	int		res;


	if ( ! sheet )
	{
		return;
	}

	res = be_commit_prefs_set ( sheet,
		mainwindow->projectGetCedFile ()->cubook,
		pref_id, pref_num, pref_charp );

	mainwindow->projectReportUpdates ( res );

	if ( be_commit_prefs_set_check ( res, sheet, change_cursor, pref_id ) )
	{
		return;
	}

	mainwindow->updateChangesChores ( 1, 1 );
}

void MainWindow :: pressOptionsCellPrefs ()
{
	CedSheet	* sheet = projectGetSheet ();
	mtPrefs		* prefs;


	if ( projectReportUpdates ( cuf_check_sheet_lock ( sheet ) ) )
	{
		return;
	}

	prefs = be_cellpref_init ( sheet, be_cellpref_changed, (void *)this );

	qexPrefsWindow ( prefs, "Cell Preferences" );


	CedCellPrefs	* cell_prefs = ced_cell_prefs_default ();


	if ( prefs && cell_prefs )
	{
		be_cellpref_bulk_import ( prefs, cell_prefs );

		fe_save_pref_window_prefs ( prefs );
	}

	mtkit_prefs_destroy ( prefs );

	be_cellpref_cleanup ( sheet );

	updateView ();
}

void MainWindow :: pressOptionsBookPrefs ()
{
	mtPrefs * prefs = be_book_prefs_init ( cedFile->cubook->book );

	qexPrefsWindow ( prefs, "Book Preferences" );
	fe_save_pref_window_prefs ( prefs );
	be_book_prefs_finish ( prefs, cedFile->cubook->book );

	mtkit_prefs_destroy ( prefs );
}

void MainWindow :: pressOptionsProgramPrefs ()
{
	qexPrefsWindow ( prefs_file (), "Preferences" );
}

void MainWindow :: pressOptionsBold ()
{
	int		res;


	if ( be_prepare_prefs_set ( projectGetSheet () ) )
	{
		return;
	}

	res = cedui_cellprefs_bold ( cedFile );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return;
	}

	projectReportUpdates ( res );
	updateChangesChores ( 1, 1 );
	be_cellpref_cleanup ( projectGetSheet () );
}

static void swFunc (
	QString		title,
	int		pref_id
	)
{
	if ( be_prepare_prefs_set ( mainwindow->projectGetSheet () ) )
	{
		return;
	}


	SwatchDialog	dialog ( title );
	int		col = dialog.getColor ();


	if ( col >= 0 )
	{
		fe_commit_prefs_set ( pref_id, col, NULL, 0, NULL );
	}

	be_cellpref_cleanup ( mainwindow->projectGetSheet () );
}

void MainWindow :: pressOptionsBackgroundColor ()
{
	swFunc ( tr ( "Background Colour" ), CEDUI_CELLPREFS_color_background );
}

void MainWindow :: pressOptionsForegroundColor ()
{
	swFunc ( tr ( "Foreground Colour" ), CEDUI_CELLPREFS_color_foreground );
}

void MainWindow :: pressOptionsBorderColor ()
{
	swFunc ( tr ( "Border Colour" ), CEDUI_CELLPREFS_border_color );
}

void MainWindow :: pressOptionsBorder ( int i )
{
	int		res;


	if ( be_prepare_prefs_set ( projectGetSheet () ) )
	{
		return;
	}

	res = cedui_cellprefs_border ( cedFile, i );

	if (	res == CU_ERROR_LOCKED_CELL ||
		res == CU_ERROR_NO_CHANGES
		)
	{
		return;
	}

	projectReportUpdates ( res );
	updateChangesChores ( 1, 1 );
	be_cellpref_cleanup ( projectGetSheet () );
}

