/*
	Copyright (C) 2013-2014 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#ifdef U_TK_QT4
	#include <mtqex4.h>
#endif

#ifdef U_TK_QT5
	#include <mtqex5.h>
#endif



#ifdef __cplusplus
	extern "C" {
#endif	//  __cplusplus


// C API

#include "be.h"



#ifdef __cplusplus
	}
#endif	//  __cplusplus



#define LABEL_RES_COL_V		0
#define LABEL_RES_COL_VP	1
#define LABEL_RES_COL_VPS	2
#define LABEL_RES_COL_S		3
#define LABEL_RES_COL_SC	4
#define LABEL_RES_COLS		5

#define LABEL_RES_ROWS		5

#define DEFAULTzoomCartogram	10
#define DEFAULTzoomDiagram	2

#define EL_TAB_VALUE( row, col )	yTable[EL_TAB_TOTAL * row + col]



enum
{
	LABEL_RESULTS_HEAD_A,
	LABEL_RESULTS_HEAD_B,
	LABEL_RESULTS_HEAD_C,
	LABEL_RESULTS_HEAD_O,
	LABEL_RESULTS_HEAD_TOTAL,

	LABEL_RES_A_V,
	LABEL_RES_A_VP,
	LABEL_RES_A_VPS,
	LABEL_RES_A_S,
	LABEL_RES_A_SC,

	LABEL_RES_B_V,
	LABEL_RES_B_VP,
	LABEL_RES_B_VPS,
	LABEL_RES_B_S,
	LABEL_RES_B_SC,

	LABEL_RES_C_V,
	LABEL_RES_C_VP,
	LABEL_RES_C_VPS,
	LABEL_RES_C_S,
	LABEL_RES_C_SC,

	LABEL_RES_O_V,
	LABEL_RES_O_VP,
	LABEL_RES_O_VPS,
	LABEL_RES_O_S,
	LABEL_RES_O_SC,

	LABEL_RES_T_V,
	LABEL_RES_T_VP,
	LABEL_RES_T_VPS,
	LABEL_RES_T_S,
	LABEL_RES_T_SC,

	LABEL_RES_TOTAL
};

enum
{
	LABEL_SWING_AB,
	LABEL_SWING_AC,
	LABEL_SWING_AO,
	LABEL_SWING_BC,
	LABEL_SWING_BO,
	LABEL_SWING_CO,

	LABEL_SWING_TOTAL
};


QT_BEGIN_NAMESPACE

class	QAction;
class	QComboBox;
class	QDoubleSpinBox;
class	QGridLayout;
class	QGroupBox;
class	QLabel;
class	QMenuBar;
class	QScrollArea;
class	QSlider;
class	QVBoxLayout;
class	QWidget;

QT_END_NAMESPACE



class	MainWindow;
class	MapArea;
class	MapAreaHolder;



extern MainWindow	* mainwindow;



class MainWindow : public QWidget
{
	Q_OBJECT

public:
	MainWindow ();
	~MainWindow ();

	void		polymapRedraw ();
	void		polymapScroll ( int dx, int dy );
	mtImage		* prepRender ( int px, int py, int pw, int ph );
	void		wheelZoom ( int x, int y, int delta );
	void		clickMap ( int x, int y );
	void		resetMapSize ();

private slots:
	void		pressFileSaveGraph ();
	void		pressFileSaveMap ();
	void		pressFileQuit ();
	void		pressEditPlotGraph ();
	void		pressEditResetSwings ();
	void		pressEditPreferences ();
	void		pressHelpAbout ();

	void		swingEleanaChanged ( int state );

	void		swingSlider0Changed ( int val );
	void		swingSlider1Changed ( int val );
	void		swingSlider2Changed ( int val );
	void		swingSlider3Changed ( int val );
	void		swingSlider4Changed ( int val );
	void		swingSlider5Changed ( int val );

	void		swingSpin0Changed ( double val );
	void		swingSpin1Changed ( double val );
	void		swingSpin2Changed ( double val );
	void		swingSpin3Changed ( double val );
	void		swingSpin4Changed ( double val );
	void		swingSpin5Changed ( double val );

	void		yearChanged ( int val );

	void		zoomCartogramChanged ( int zoom );
	void		zoomDiagramChanged ( int zoom );
	void		zoomMapChanged ( int zoom );

private:
	void		createMenus ();
	void		menuInit (
				QAction ** action,
				const char * text,
				const char * shortcut,
				const char * icon
				);

	void		clearSeatTable ();
	void		loadPartyNames ();

	void		mapRedraw ();
	void		diagramRedraw ();

	void		projectClearAll ();
	int		projectLoad ( const char * filename );
	void		projectRegister ( const char * filename );
	int		projectSetFont ( char * filename, int size );

	void		purgeIndexLoaded ();
	void		purgeDataLoaded ();

	void		resetMapZoomPos ();

	void		swingSheetUpdate ();
	void		swingSpinChanged ( int i, double val );
	void		swingSliderChanged ( int i, int val );

	void		updateTitlebar ();


	QMenuBar	* menuBar;
	QComboBox	* comboYear;
	QGridLayout	* gridSeat;
	QVBoxLayout	* seatHolderVbox;
	QSlider		* sliderMapZoom;

	qexImage	* cartogramWidget,
			* diagramWidget,
			* graphWidget
			;
	QLabel		* labelResults [ LABEL_RES_TOTAL ],
			* labelSwing [ LABEL_SWING_TOTAL ]
			;
	QSlider		* sliderSwing [ LABEL_SWING_TOTAL ];
	QDoubleSpinBox	* spinSwing [ LABEL_SWING_TOTAL ];

	MapAreaHolder	* polymapScrollArea;
	MapArea		* mapArea;

	double		swingVal [ LABEL_SWING_TOTAL ],
			zoomMap;

	int		swingEleana;

	mtImage		* diagramImage;
	eleanaINDEX	* eleanaIndex;

	CedSheet	* ySheetFull;
	double		* yTable;
	int		ySeats,
			yPartySeats [4];
	mtTree		* ySeatNames,
			* yPolymapTree;
	CedIndex	* ySeatIndex;

	mtFont		* font;
	int		fontW,
			fontH
			;

	char		memFilename [ 2048 ],
			partyName [ 4 ][ PART_NAME_SIZE ]
			;
};

class MapAreaHolder : public QScrollArea
{
	Q_OBJECT

private:
	void		resizeEvent ( QResizeEvent * event );
};

class MapArea : public QWidget
{
	Q_OBJECT

public:
	MapArea ();

private:
	void		paintEvent ( QPaintEvent * event );
	void		mousePressEvent ( QMouseEvent * event );
	void		mouseMoveEvent ( QMouseEvent * event );
	void		mouseEventRouter ( QMouseEvent * event, int caller );
	void		wheelEvent ( QWheelEvent * event );

	int		oldMouseX,
			oldMouseY
			;
};

