/*
	Copyright (C) 2013-2014 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "tk_qt4.h"



void MainWindow :: swingSliderChanged ( int i, int val )
{
	swingVal [ i ] = (double)val;

	spinSwing [ i ]->blockSignals ( true );
	spinSwing [ i ]->setValue ( (double)val );
	spinSwing [ i ]->blockSignals ( false );

	swingSheetUpdate ();
}

void MainWindow :: swingSpinChanged ( int i, double val )
{
	swingVal [ i ] = val;

	sliderSwing [ i ]->blockSignals ( true );
	sliderSwing [ i ]->setValue ( (int)val );
	sliderSwing [ i ]->blockSignals ( false );

	swingSheetUpdate ();
}

#define SPINSLIDER_FUNC( XXX ) \
void MainWindow :: swingSlider ## XXX ## Changed ( int val ) \
{ \
	swingSliderChanged ( XXX, val ); \
} \
void MainWindow :: swingSpin ## XXX ## Changed ( double val ) \
{ \
	swingSpinChanged ( XXX, val ); \
} \

SPINSLIDER_FUNC ( 0 )
SPINSLIDER_FUNC ( 1 )
SPINSLIDER_FUNC ( 2 )
SPINSLIDER_FUNC ( 3 )
SPINSLIDER_FUNC ( 4 )
SPINSLIDER_FUNC ( 5 )

void MainWindow :: swingEleanaChanged ( int state )
{
	double		new_swing [ EL_SWING_TOTAL ];
	int		i;


	eleana_swingset_prepare ( yTable, ySeats, swingVal, new_swing );

	if ( state )
	{
		swingEleana = 1;

		// Moving from Uniform to Eleana swing
		eleana_swingset_uniform_to_eleana ( swingVal, new_swing );
	}
	else
	{
		swingEleana = 0;

		// Moving from Eleana to Uniform swing
		eleana_swingset_eleana_to_uniform ( swingVal, new_swing );
	}

	if ( yTable )		// Set spinsliders to new values
	{
		for ( i = 0; i < EL_SWING_TOTAL; i++ )
		{
			spinSwing [ i ]->setValue ( new_swing [ i ] );
		}
	}

	swingSheetUpdate ();
}

void MainWindow :: zoomCartogramChanged ( int zoom )
{
	cartogramWidget->setZoom ( zoom );
}

void MainWindow :: zoomDiagramChanged ( int zoom )
{
	diagramWidget->setZoom ( zoom );
}

void MainWindow :: zoomMapChanged ( int zoom )
{
	int		nv_h,
			nv_v,
			w,
			h;
	double		cx = -1,
			cy = -1,
			aw,
			ah,
			x,
			y;


	aw = polymapScrollArea->viewport ()->width ();
	ah = polymapScrollArea->viewport ()->height ();

	x = polymapScrollArea->horizontalScrollBar ()->value ();
	y = polymapScrollArea->verticalScrollBar ()->value ();

	cx = x - aw + aw/2;
	cy = y - ah + ah/2;
	cx /= (mapArea->width () - 2 * aw);
	cy /= (mapArea->height () - 2 * ah);

	zoomMap = ( (double)zoom ) / 100;

	if ( cx > 1 ) cx = 1;
	if ( cx < 0 ) cx = 0;
	if ( cy > 1 ) cy = 1;
	if ( cy < 0 ) cy = 0;

	w = rint ( zoomMap * POLYMAP_WIDTH );
	h = rint ( zoomMap * POLYMAP_HEIGHT );

	nv_h = rint ( cx * w - aw / 2 );
	nv_v = rint ( cy * h - ah / 2 );

	mapArea->setGeometry ( 0, 0, w + aw * 2, h + ah * 2 );

	polymapScrollArea->horizontalScrollBar ()->setValue ( nv_h + aw );
	polymapScrollArea->verticalScrollBar ()->setValue ( nv_v + ah );
}

void MainWindow :: yearChanged ( int val )
{
	CedSheet	* full;
	mtTree		* polymap,
			* seat_names;
	CedIndex	* seat_index;
	double		* table;
	int		seats;


	if ( ! eleanaIndex )
	{
		return;
	}

	if ( eleana_index_record_load ( eleanaIndex, val, 0,
		&full, &polymap, &table, &seats, &seat_names, &seat_index ) )
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			tr ( "Unable to change year" ) );

		return;
	}

	purgeDataLoaded ();		// Remove old year's data

	ySheetFull = full;
	yTable = table;
	ySeats = seats;
	yPolymapTree = polymap;
	ySeatNames = seat_names;
	ySeatIndex = seat_index;

	loadPartyNames ();

	prefs_set_int ( PREFS_LAST_YEAR, val );

	swingSheetUpdate ();

	graphWidget->setImage ( NULL );

	clearSeatTable ();
}

