/*
	Copyright (C) 2013-2014 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "tk_qt4.h"

#include "icon.xpm"



MainWindow	* mainwindow;



MainWindow :: MainWindow ()
	:
	zoomMap		( 1.0 ),
	swingEleana	( 0 ),
	diagramImage	(),
	eleanaIndex	(),

	ySheetFull	(),
	yTable		(),
	ySeats		( 0 ),
	ySeatNames	(),
	yPolymapTree	(),
	ySeatIndex	(),

	font		(),
	fontW		( 0 ),
	fontH		( 0 )
{
	QWidget		* widget,
			* tab,
			* scrollAreaContent;
	QVBoxLayout	* layout,
			* vbox;
	QSplitter	* split;
	QHBoxLayout	* row;
	QTabWidget	* tabWidget;
	QGroupBox	* groupBox;
	QScrollArea	* scrollArea;
	QGridLayout	* grid;
	QSlider		* slider;


	mainwindow = this;
	memFilename[0] = 0;


	// Preparation

	prefs_init ();

	mtQEX :: fpickInitPrefs ( prefs_file (), (char *)PREFS_FPICK_PRE );
	mtQEX :: prefsInitPrefs ( prefs_file () );

	prefs_load ();

	ced_init ();
	mtpixel_init ();

	diagramImage = mtpixel_image_load ( DATA_DIR "diagrami.png" );


	// Widgets

	setWindowTitle ( VERSION );
	this->setWindowIcon ( QPixmap ( icon_xpm ) );

	layout = new QVBoxLayout ( this );
	layout->setMargin ( 0 );

	split = new QSplitter ( Qt::Horizontal );
	layout->addWidget ( split );

	// Left split area

	widget = new QWidget;
	layout = new QVBoxLayout;
	layout->setMargin ( 0 );
	layout->setSpacing ( 0 );
	widget->setLayout ( layout );
	split->addWidget ( widget );

	row = new QHBoxLayout;
	row->setMargin ( 0 );
	row->setSpacing ( 0 );
	layout->addLayout ( row );

	menuBar = new QMenuBar;
	row->addWidget ( menuBar );

	scrollArea = new QScrollArea;
	scrollArea->setWidgetResizable ( true );
	layout->addWidget ( scrollArea );

	scrollAreaContent = new QWidget;
	layout = new QVBoxLayout ( scrollAreaContent );

	groupBox = new QGroupBox ( tr ( "Year" ) );
	layout->addWidget ( groupBox );

	comboYear = new QComboBox;
	vbox = new QVBoxLayout;
	groupBox->setLayout ( vbox );
	vbox->addWidget ( comboYear );

	connect ( comboYear, SIGNAL ( currentIndexChanged ( int ) ),
		this, SLOT ( yearChanged ( int ) ) );

	groupBox = new QGroupBox ( tr ( "Results" ) );
	layout->addWidget ( groupBox );

	grid = new QGridLayout;
	groupBox->setLayout ( grid );


	int		i,
			j;
	QLabel		* label;
	const char	* tab_titles [] = {
				"Party",
				"Votes",
				"Share",
				"Swing",
				"Seats",
				"Change"
				},
			* party_txt [] = { "A", "B", "C", "O", "Total" },
			* party_swing_txt [ EL_SWING_TOTAL ] = {
				"A->B",
				"A->C",
				"A->O",
				"B->C",
				"B->O",
				"C->O"
				};


	for ( i = 0; i <= LABEL_RES_COLS; i++ )
	{
		label = new QLabel ( tr ( tab_titles [ i ] ) );
		label->setAlignment ( Qt::AlignHCenter | Qt::AlignVCenter );

		grid->addWidget ( label, 0, i );
	}

	for ( i = 0; i < LABEL_RES_ROWS; i++ )
	{
		label = new QLabel ( tr ( party_txt [ i ] ) );
		label->setAlignment ( Qt::AlignHCenter | Qt::AlignVCenter );

		labelResults [ LABEL_RESULTS_HEAD_A + i ] = label;

		grid->addWidget ( label, i + 1, 0 );

		for ( j = 1; j <= LABEL_RES_COLS; j++ )
		{

			label = new QLabel ( "-" );
			label->setAlignment ( Qt::AlignHCenter |
				Qt::AlignVCenter );

			labelResults [ LABEL_RES_A_V + i * LABEL_RES_COLS +
				j - 1 ] = label;

			grid->addWidget ( label, i + 1, j );
		}
	}

	groupBox = new QGroupBox ( tr ( "Seat (Ctrl + Click Map)" ) );
	groupBox->setSizePolicy ( QSizePolicy ( QSizePolicy::Preferred,
		QSizePolicy::Expanding ) );
	layout->addWidget ( groupBox );

	gridSeat = new QGridLayout ( groupBox );

	groupBox = new QGroupBox ( tr ( "Swing" ) );
	layout->addWidget ( groupBox );

	vbox = new QVBoxLayout;
	groupBox->setLayout ( vbox );

	grid = new QGridLayout;
	vbox->addLayout ( grid );

	for ( i = 0; i < EL_SWING_TOTAL; i++ )
	{
		label = new QLabel ( tr ( party_swing_txt [ i ] ) );
		grid->addWidget ( label, i, 0 );

		labelSwing [ i ] = label;

		sliderSwing [ i ] = new QSlider ( Qt::Horizontal );
		grid->addWidget ( sliderSwing [ i ], i, 1 );

		sliderSwing [ i ]->setRange ( -90, 90 );
		sliderSwing [ i ]->setSingleStep ( 1 );

		spinSwing [ i ] = new QDoubleSpinBox;
		grid->addWidget ( spinSwing [ i ], i, 2 );

		spinSwing [ i ]->setRange ( -90.0, 90.0 );
		spinSwing [ i ]->setSingleStep ( 0.1 );
		spinSwing [ i ]->setDecimals ( 2 );

		swingVal [ i ] = 0.0;
	}

#define CONNECT_SPINSLIDER( XXX ) \
	connect ( spinSwing [ XXX ], SIGNAL ( valueChanged ( double ) ), \
		this, SLOT ( swingSpin ## XXX ## Changed ( double ) ) ); \
	connect ( sliderSwing [ XXX ], SIGNAL ( valueChanged ( int ) ), \
		this, SLOT ( swingSlider ## XXX ## Changed ( int ) ) ); \

	CONNECT_SPINSLIDER ( 0 )
	CONNECT_SPINSLIDER ( 1 )
	CONNECT_SPINSLIDER ( 2 )
	CONNECT_SPINSLIDER ( 3 )
	CONNECT_SPINSLIDER ( 4 )
	CONNECT_SPINSLIDER ( 5 )

	QCheckBox * cb = new QCheckBox ( tr ( "Eleana Swing" ) );
	vbox->addWidget ( cb );
	connect ( cb, SIGNAL ( stateChanged ( int ) ),
		this, SLOT ( swingEleanaChanged ( int ) ) );

	scrollArea->setWidget ( scrollAreaContent );

	// Right split area

	tabWidget = new QTabWidget;
	split->addWidget ( tabWidget );

	tab = new QWidget;
	tabWidget->addTab ( tab, tr ( "Map" ) );

	layout = new QVBoxLayout;
	layout->setMargin ( 0 );
	layout->setSpacing ( 0 );
	tab->setLayout ( layout );

	slider = new QSlider ( Qt::Horizontal );
	layout->addWidget ( slider );

	slider->setRange ( 30, 2500 );
	slider->setValue ( 100 * zoomMap );
	sliderMapZoom = slider;

	connect ( slider, SIGNAL ( valueChanged ( int ) ),
		this, SLOT ( zoomMapChanged ( int ) ) );

	polymapScrollArea = new MapAreaHolder;
	layout->addWidget ( polymapScrollArea );

	mapArea = new MapArea;
	polymapScrollArea->setWidget ( mapArea );

	// Cartogram Tab

	tab = new QWidget;
	tabWidget->addTab ( tab, tr ( "Cartogram" ) );

	layout = new QVBoxLayout;
	layout->setMargin ( 0 );
	layout->setSpacing ( 0 );
	tab->setLayout ( layout );

	slider = new QSlider ( Qt::Horizontal );
	layout->addWidget ( slider );

	slider->setRange ( 1, 32 );
	slider->setValue ( DEFAULTzoomCartogram );

	connect ( slider, SIGNAL ( valueChanged ( int ) ),
		this, SLOT ( zoomCartogramChanged ( int ) ) );

	cartogramWidget = new qexImage;
	cartogramWidget->setImage ( mtpixel_image_new_rgb ( MAP_WIDTH,
		MAP_HEIGHT ) );
	cartogramWidget->setZoom ( DEFAULTzoomCartogram );
	layout->addWidget ( cartogramWidget );

	// Diagram Tab

	tab = new QWidget;
	tabWidget->addTab ( tab, tr ( "Diagram" ) );

	layout = new QVBoxLayout;
	layout->setMargin ( 0 );
	layout->setSpacing ( 0 );
	tab->setLayout ( layout );

	slider = new QSlider ( Qt::Horizontal );
	layout->addWidget ( slider );

	slider->setRange ( 1, 16 );
	slider->setValue ( DEFAULTzoomDiagram );

	connect ( slider, SIGNAL ( valueChanged ( int ) ),
		this, SLOT ( zoomDiagramChanged ( int ) ) );

	diagramWidget = new qexImage;
	diagramWidget->setImage ( mtpixel_image_duplicate ( diagramImage ) );
	diagramWidget->setZoom ( DEFAULTzoomDiagram );
	layout->addWidget ( diagramWidget );

	// Graph Tab

	tab = new QWidget;
	tabWidget->addTab ( tab, tr ( "Graph" ) );

	layout = new QVBoxLayout;
	layout->setMargin ( 0 );
	layout->setSpacing ( 0 );
	tab->setLayout ( layout );

	graphWidget = new qexImage;
	layout->addWidget ( graphWidget );

	createMenus ();

	// Final details

	setMinimumSize ( 160, 160 );

	setGeometry ( prefs_get_int ( PREFS_WINDOW_X ),
		prefs_get_int ( PREFS_WINDOW_Y ),
		prefs_get_int ( PREFS_WINDOW_W ),
		prefs_get_int ( PREFS_WINDOW_H ) );

	if ( projectSetFont (
		prefs_get_string ( PREFS_FONT_FILENAME ),
		prefs_get_int ( PREFS_FONT_SIZE ) )
		)
	{
		projectSetFont ( NULL, 16 );
	}

	updateTitlebar ();

	projectLoad ( DATA_DIR "eleana_index.zip" );

	if ( ! eleanaIndex )
	{
		projectClearAll ();	// Start with empty file structure
	}

	// Make split sensible
	QList<int> ss = split->sizes ();
	ss.replace ( 0, 1 );
	ss.replace ( 1, 1 );
	split->setSizes ( ss );

	show ();

	resetMapZoomPos ();
}

MainWindow :: ~MainWindow ()
{
	prefs_set_int ( PREFS_WINDOW_X, geometry().x () );
	prefs_set_int ( PREFS_WINDOW_Y, geometry().y () );
	prefs_set_int ( PREFS_WINDOW_W, geometry().width () );
	prefs_set_int ( PREFS_WINDOW_H, geometry().height () );

	prefs_save ();
	prefs_close ();

	mtpixel_image_destroy ( diagramImage );
	diagramImage = NULL;

	mtpixel_font_destroy ( font );
	font = NULL;

	purgeDataLoaded ();
	purgeIndexLoaded ();

	mtpixel_quit ();
}

int main (
	int		argc,
	char		** argv
	)
{
	be_cline ( argc, argv );


	QApplication	app ( argc, argv );
	MainWindow	window;


	return app.exec ();
}

