/*
	Copyright (C) 2013-2014 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "tk_qt4.h"



void MainWindow :: menuInit (
	QAction		** action,
	const char	* text,
	const char	* shortcut,
	const char	* icon
	)
{
	if ( icon )
	{
		action[0] = new QAction ( QIcon :: fromTheme ( icon ),
			tr ( text ), this );
		action[0]->setIconVisibleInMenu ( true );
	}
	else
	{
		action[0] = new QAction ( tr ( text ), this );
	}

	if ( shortcut )
	{
		action[0]->setShortcut ( tr ( shortcut ) );
	}
}

void MainWindow :: pressFileSaveGraph ()
{
	mtImage		* image = graphWidget->getImage ();


	if ( ! image )
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			tr ( "No graph to save." ) );

		return;
	}


	QString filename = QFileDialog::getSaveFileName ( this,
		tr ( "Save Graph File" ),
		mtQEX::qstringFromC ( prefs_get_string ( PREFS_LAST_DIR ) ) +
			"/graph.png",
		tr ( "Images (*.png)" ), NULL,
		QFileDialog::DontUseNativeDialog );


	if ( ! filename.isEmpty () )
	{
		if ( mtpixel_image_save ( image, filename.toUtf8 ().data (),
			MTPIXEL_FILE_TYPE_PNG, 5 ) )
		{
			QMessageBox :: critical ( this, tr ( "Error" ),
				tr ( "Unable to save graph." ) );

			return;
		}

		be_file_to_last_dir ( filename.toUtf8 ().data () );
	}
}

void MainWindow :: pressFileSaveMap ()
{
	QString filename = QFileDialog::getSaveFileName ( this,
		tr ( "Save Map File" ),
		mtQEX::qstringFromC ( prefs_get_string ( PREFS_LAST_DIR ) ) +
			"/map.png",
		tr ( "Images (*.png)" ), NULL,
		QFileDialog::DontUseNativeDialog );


	if ( ! filename.isEmpty () )
	{
		if ( be_save_polymap ( filename.toUtf8 ().data (),
			yPolymapTree, yTable, zoomMap ) )
		{
			QMessageBox :: critical ( this, tr ( "Error" ),
				tr ( "Unable to save map." ) );

			return;
		}

		be_file_to_last_dir ( filename.toUtf8 ().data () );
	}
}

void MainWindow :: pressFileQuit ()
{
	close ();
}

void MainWindow :: pressEditPlotGraph ()
{
	graphWidget->setImage ( be_plot_graph ( yTable, font, ySeats ) );
}

void MainWindow :: pressEditResetSwings ()
{
	int		i;


	for ( i = LABEL_SWING_AB; i <= LABEL_SWING_CO; i++ )
	{
		sliderSwing [ i ]->setValue ( 0 );
	}
}

void MainWindow :: pressEditPreferences ()
{
	qexPrefsWindow ( prefs_file (), "Preferences" );
}

void MainWindow :: pressHelpAbout ()
{
	QMessageBox :: about ( this, tr ( "About" ),
		tr ( VERSION"\n"
	"\n"
	"Copyright (C) 2009-2014 Mark Tyler.\n"
	"\n"
	"This program is free software: you can redistribute it and/or modify "
	"it under the terms of the GNU General Public License as published by "
	"the Free Software Foundation, either version 3 of the License, or "
	"(at your option) any later version.\n"
	"\n"
	"This program is distributed in the hope that it will be useful, "
	"but WITHOUT ANY WARRANTY; without even the implied warranty of "
	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
	"GNU General Public License for more details.\n"
	"\n"
	"You should have received a copy of the GNU General Public License "
	"along with this program.  If not, see http://www.gnu.org/licenses/\n"
		) );
}

void MainWindow :: createMenus ()
{
	QAction
			* actFileSaveGraph,
			* actFileSaveMap,
			* actFileQuit,

			* actEditPlotGraph,
			* actEditResetSwings,
			* actEditPreferences,

			* actHelpAbout
			;


#define MENFU( A, B, C, D ) \
	menuInit ( &act ## A , B, C, D ); \
	connect ( act ## A, SIGNAL ( triggered () ), this, \
		SLOT ( press ## A () ) );


MENFU ( FileSaveGraph,		"Save Graph Image ...",	NULL,	NULL )
MENFU ( FileSaveMap,		"Save Map Image ...",	NULL,	NULL )
MENFU ( FileQuit,		"Quit",			"Ctrl+Q", "application-exit" )

MENFU ( EditPlotGraph,		"Plot Graph",		"F6",	NULL )
MENFU ( EditResetSwings,	"Reset Swings",		"F9",	NULL )
MENFU ( EditPreferences,	"Preferences",		"Ctrl+P", "preferences-other" )

MENFU ( HelpAbout,		"About ...",		"F1", "help-about" )


	QMenu * menuFile = menuBar->addMenu ( tr ( "&File" ) );
	menuFile->setTearOffEnabled ( true );
	menuFile->addAction ( actFileSaveGraph );
	menuFile->addAction ( actFileSaveMap );
	menuFile->addSeparator ();
	menuFile->addAction ( actFileQuit );

	QMenu * menuEdit = menuBar->addMenu ( tr ( "&Edit" ) );
	menuEdit->setTearOffEnabled ( true );

	menuEdit->addAction ( actEditPlotGraph );
	menuEdit->addAction ( actEditResetSwings );
	menuEdit->addSeparator ();
	menuEdit->addAction ( actEditPreferences );

	QMenu * menuHelp = menuBar->addMenu ( tr ( "&Help" ) );

	menuHelp->addAction ( actHelpAbout );
}

