/*
	Copyright (C) 2013-2014 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "tk_qt4.h"



void MainWindow :: projectClearAll ()
{
	purgeDataLoaded ();

	memFilename[0] = 0;

	updateTitlebar ();
}

void MainWindow :: projectRegister ( const char * filename )
{
	if ( ! filename )
	{
		return;
	}

	mtkit_strncpy0 ( memFilename, filename, PATHBUF );
	updateTitlebar ();
}

int MainWindow :: projectLoad ( const char * filename )
{
	int		i = 0;
	char		real_filename [ PATHBUF ];
	eleanaINDEX	* newindex;
	CedCell		* cell;


	if ( ! filename || ! filename[0] )
	{
		return 1;
	}

	if ( filename[0] != MTKIT_DIR_SEP )
	{
		// Expand the filename to include the full path
		if ( getcwd ( real_filename, PATHBUF - 1 ) )
		{
			i = strlen ( real_filename );
			real_filename[i] = MTKIT_DIR_SEP;
			i++;
		}
	}

	real_filename[i] = 0;
	mtkit_strnncat ( real_filename, filename, PATHBUF );

	newindex = eleana_index_open ( real_filename );
	if ( ! newindex )
	{
		QString		tmp = mtQEX::qstringFromC ( real_filename );


		QMessageBox :: critical ( this, tr ( "Error" ),
			QString ( tr ( "Unable to load index file %1." )
			).arg ( tmp ) );

		return 1;
	}

	// Index was loaded correctly so purge all other data
	purgeIndexLoaded ();
	purgeDataLoaded ();

	eleanaIndex = newindex;

	// Remove the old years in the picker
	comboYear->clear ();

	if ( 0 != prefs_get_int ( PREFS_LAST_YEAR ) )
	{
		comboYear->blockSignals ( true );
	}

	for ( i = ELEANA_INDEX_ROW_START; ; i++ )
	{
		cell = ced_sheet_get_cell ( newindex->sheet_index, i,
			ELEANA_INDEX_COL_YEAR );

		if ( ! cell )
		{
			break;
		}

		comboYear->addItem ( mtQEX::qstringFromC ( cell->text ) );
	}

	pressEditResetSwings ();

	if ( 0 != prefs_get_int ( PREFS_LAST_YEAR ) )
	{
		comboYear->blockSignals ( false );

		// Default to the previous year that was used
		comboYear->setCurrentIndex ( prefs_get_int ( PREFS_LAST_YEAR ));
	}

	projectRegister ( real_filename );

	resetMapZoomPos ();

	return 0;			// Success
}

int MainWindow :: projectSetFont (
	char		* filename,
	int		size
	)
{
	if ( be_font_new ( filename, size, &font, &fontW, &fontH ) )
	{
		QMessageBox :: critical ( this, tr ( "Error" ),
			QString ( tr ( "Unable to open font file %1 size %2." )
			).arg ( filename ).arg ( size ) );

		return 1;
	}

	return 0;			// Success
}

void MainWindow :: updateTitlebar ()
{
	QString		qs = mtQEX::qstringFromC ( memFilename );


	if ( qs.isEmpty () )
	{
		qs = tr ( "Untitled" );
	}

	setWindowTitle ( QString ( VERSION " - " ) + qs );
}

void MainWindow :: purgeIndexLoaded ()
{
	eleana_index_close ( eleanaIndex );
	eleanaIndex = NULL;
}

void MainWindow :: purgeDataLoaded ()
{
	eleana_index_record_close ( ySheetFull, yPolymapTree,
		yTable, ySeatNames, ySeatIndex );

	ySheetFull = NULL;
	yTable = NULL;
	yPolymapTree = NULL;
	ySeatNames = NULL;
	ySeatIndex = NULL;
}

void MainWindow :: loadPartyNames ()
{
	const char	* names[4] = { "CON", "LAB", "LIB", "OTH" };
	char		txt[128];
	int		j,
			swlab[EL_SWING_TOTAL][3] = {
				{ 0, 1, LABEL_SWING_AB },
				{ 0, 2, LABEL_SWING_AC },
				{ 0, 3, LABEL_SWING_AO },
				{ 1, 2, LABEL_SWING_BC },
				{ 1, 3, LABEL_SWING_BO },
				{ 2, 3, LABEL_SWING_CO }
				};


	for ( j = 0; j < 4; j++ )
	{
		labelResults [ LABEL_RESULTS_HEAD_A + j ]->setText (
			mtQEX::qstringFromC ( names[j] ) );

		snprintf ( partyName[j], PART_NAME_SIZE, "%s",
			names[j] );

		partyName[j][PART_NAME_SIZE - 1] = 0;
	}

	for ( j = 0; j < EL_SWING_TOTAL; j++ )
	{
		snprintf ( txt, sizeof ( txt ), "%s->%s", names[ swlab[j][0] ],
			names[ swlab[j][1] ] );

		txt [ sizeof ( txt ) - 1 ] = 0;

		labelSwing [ swlab[j][2] ]->setText ( mtQEX::qstringFromC (
			txt ) );
	}
}

void MainWindow :: swingSheetUpdate ()
{
	double		p1,
			p2;
	char		txt[128];
	int		i,
			j,
			tab[4][3] = {
			{ EL_TAB_PROJ_A, EL_TAB_PROJ_A_SEAT, EL_TAB_A },
			{ EL_TAB_PROJ_B, EL_TAB_PROJ_B_SEAT, EL_TAB_B },
			{ EL_TAB_PROJ_C, EL_TAB_PROJ_C_SEAT, EL_TAB_C },
			{ EL_TAB_PROJ_O, EL_TAB_PROJ_O_SEAT, EL_TAB_O }
			};


	if ( ! yTable )
	{
		return;
	}

	if ( swingEleana )
	{
		eleana_calculate_eleana_swing ( yTable, ySeats,
			swingVal, yPartySeats );
	}
	else
	{
		eleana_calculate_uniform_swing ( yTable, ySeats,
			swingVal, yPartySeats );
	}

	for ( i = 0; i < 4; i++ )
	{
		j = rint ( EL_TAB_VALUE ( ySeats, tab[i][0] ) );
		sprintf ( txt, "%i", j );

		labelResults [ LABEL_RES_A_V + i * LABEL_RES_COLS +
			LABEL_RES_COL_V]->setText ( txt );

		p1 = 100 * ( (double)j ) /
			EL_TAB_VALUE ( ySeats, EL_TAB_TURNOUT );

		sprintf ( txt, "%.1f%%", p1 );

		labelResults [ LABEL_RES_A_V + i * LABEL_RES_COLS +
			LABEL_RES_COL_VP]->setText ( txt );

		p2 = p1 - 100 * EL_TAB_VALUE ( ySeats, tab[i][2] ) /
			EL_TAB_VALUE ( ySeats, EL_TAB_TURNOUT );

		if ( p2 < 0.001 && p2 > -0.001 )
		{
			p2 = 0;		// Stops irritating -0.0
		}

		sprintf ( txt, "%.1f%%", p2 );

		labelResults [ LABEL_RES_A_V + i * LABEL_RES_COLS +
			LABEL_RES_COL_VPS]->setText ( txt );

		j = EL_TAB_VALUE ( ySeats, tab[i][1] );

		sprintf ( txt, "%i", j );

		labelResults [ LABEL_RES_A_V + i * LABEL_RES_COLS +
			LABEL_RES_COL_S]->setText ( txt );
	}

	sprintf ( txt, "%i", ySeats );
	labelResults [ LABEL_RES_T_S ]->setText ( txt );

	j = rint ( EL_TAB_VALUE ( ySeats, EL_TAB_PROJ_A ) +
		EL_TAB_VALUE ( ySeats, EL_TAB_PROJ_B ) +
		EL_TAB_VALUE ( ySeats, EL_TAB_PROJ_C ) +
		EL_TAB_VALUE ( ySeats, EL_TAB_PROJ_O ) );

	sprintf ( txt, "%i", j );
	labelResults [ LABEL_RES_T_V ]->setText ( txt );

	i = EL_TAB_VALUE ( ySeats, EL_TAB_TURNOUT );
	sprintf ( txt, "%.1f%%", 100 * ( (double)j ) / i );
	labelResults [ LABEL_RES_T_VP ]->setText ( txt );

	for ( i = 0; i < 4; i++ )
	{
		sprintf ( txt, "%i", (int)EL_TAB_VALUE ( ySeats,
			EL_TAB_PROJ_A_SEAT + i ) - yPartySeats[i] );

		labelResults [ LABEL_RES_A_SC + i * LABEL_RES_COLS]->setText (
			txt );
	}

	mapRedraw ();

	if ( diagramWidget->getImage () )
	{
		int		w, h;


		w = diagramWidget->getImage ()->width;
		h = diagramWidget->getImage ()->height;

		eleana_calculate_diagram ( yTable, ySeats, w - 1, h - 1 );

		diagramRedraw ();
	}
}

void MainWindow :: diagramRedraw ()
{
	unsigned char	* dest,
			* d1;
	int		i,
			x,
			y,
			w,
			h;


	if (	! diagramWidget->getImage ()	||
		! diagramImage			||
		! diagramImage->channel [ MTPIXEL_CHANNEL_IMAGE ]
		)
	{
		return;
	}

	w = diagramWidget->getImage ()->width;
	h = diagramWidget->getImage ()->height;
	i = w * h;
	d1 = diagramWidget->getImage ()->channel [ MTPIXEL_CHANNEL_IMAGE ];
	memcpy ( d1, diagramImage->channel [ MTPIXEL_CHANNEL_IMAGE ], i );

	for ( i = 0; i < ySeats; i++ )
	{
		x = EL_TABLE_VALUE ( yTable, i, EL_TAB_DIAGRAM_X ) + 0.5;
		y = EL_TABLE_VALUE ( yTable, i, EL_TAB_DIAGRAM_Y ) + 0.5;

		if (	x < 0	||
			y < 0	||
			x >= w	||
			y >= h
			)
		{
			continue;
		}

		dest = d1 + (x + y * w);
		*dest++ = 0;
	}

	diagramWidget->update ();
}

void MainWindow :: mapRedraw ()
{
	unsigned char	pal[4][3] = {
			{ 0, 0, 255 },
			{ 255, 0, 0 },
			{ 255, 255, 0 },
			{ 200, 200, 200 }
			},
			c,
			* dest,
			* d1;
	int		r,
			x,
			y;


	mapArea->update ();

	if (	! cartogramWidget->getImage ()	||
		! yTable
		)
	{
		return;
	}

	d1 = cartogramWidget->getImage ()->channel [ MTPIXEL_CHANNEL_IMAGE ];
	if ( ! d1 )
	{
		return;
	}

	// Clear the old rubbish
	memset ( d1, 0, MAP_WIDTH * MAP_HEIGHT * 3 );

	for ( r = 0; r < ySeats; r++ )
	{
		x = EL_TAB_VALUE ( r, EL_TAB_MAP_X );
		y = EL_TAB_VALUE ( r, EL_TAB_MAP_Y );
		c = EL_TAB_VALUE ( r, EL_TAB_PROJ_WINNER ) - 1;

		if ( x < 0 || x >= MAP_WIDTH )
		{
			continue;
		}

		if ( y < 0 || y >= MAP_HEIGHT )
		{
			continue;
		}

		if ( c > 3 )
		{
			continue;
		}

		dest = d1 + 3 * (x + y * MAP_WIDTH);
		*dest++ = pal[c][0];
		*dest++ = pal[c][1];
		*dest++ = pal[c][2];
	}

	cartogramWidget->update ();
}

void MainWindow :: clearSeatTable ()
{
	QLayoutItem	* item;


	while ( ( item = gridSeat->takeAt ( 0 ) ) != 0 )
	{
		QWidget		* widget;


		if ( ( widget = item->widget () )  )
		{
			delete widget;
		}

		delete item;
	}

	// Force full shrink of group box
	gridSeat->update ();
}

