/*
	Copyright (C) 2013-2014 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#ifdef U_TK_QT4
	#include <mtqex4.h>
#endif

#ifdef U_TK_QT5
	#include <mtqex5.h>
#endif



#ifdef __cplusplus
	extern "C" {
#endif	//  __cplusplus


// C API

#include "be.h"



#ifdef __cplusplus
	}
#endif	//  __cplusplus



QT_BEGIN_NAMESPACE

class	QAction;
class	QActionGroup;
class	QCloseEvent;
class	QMainWindow;
class	QMenu;
class	QMouseEvent;
class	QPaintEvent;
class	QResizeEvent;
class	QString;
class	QWidget;

QT_END_NAMESPACE



class	MainWindow;
class	jigsawCanvas;



class jigsawCanvas : public QWidget
{
	Q_OBJECT

public:
	jigsawCanvas	();
	~jigsawCanvas	();

	void		setJigsawTable ( int table );
	void		setJigsawMesh ( int hori, int vert );
	void		setJigsawImage ( QString filename );
	void		joinJigsaw ();
	void		scatterJigsaw ();
	void		saveJigsaw ();

private:
	void		paintEvent ( QPaintEvent * event );
	void		resizeEvent ( QResizeEvent * event );
	void		mousePressEvent ( QMouseEvent * event );
	void		mouseMoveEvent ( QMouseEvent * event );
	void		mouseReleaseEvent ( QMouseEvent * event );

	int
			clickPieceID,	// -1 = Nothing selected
			clickX,
			clickY,
			currentTable
			;

	mtJigsaw	* jigsaw;
};



class MainWindow : public QMainWindow
{
	Q_OBJECT

public:
	MainWindow	();
	~MainWindow	();

	int		isBusy ();	// 0 = idle
	void		setMesh ( int hori, int vert );

protected:
	void		closeEvent ( QCloseEvent * event );

private slots:
	void		pressHelpAbout ();
	void		pressJigsaw1 ();
	void		pressJigsaw2 ();
	void		pressJigsaw3 ();
	void		pressJigsaw4 ();
	void		pressJigsaw5 ();
	void		pressJigsaw6 ();
	void		pressJigsaw7 ();
	void		pressJigsaw8 ();
	void		pressJigsaw9 ();
	void		pressJigsaw10 ();
	void		pressOptionsNew ();
	void		pressOptionsJoin ();
	void		pressOptionsScatter ();
	void		pressOptionsOpenImage ();
	void		pressOptionsFullscreen ();
	void		pressOptionsPrefs ();
	void		pressOptionsQuit ();

private:
	QAction
			* actHelpAbout,
			* actJigsaw1,
			* actJigsaw2,
			* actJigsaw3,
			* actJigsaw4,
			* actJigsaw5,
			* actJigsaw6,
			* actJigsaw7,
			* actJigsaw8,
			* actJigsaw9,
			* actJigsaw10,
			* actOptionsNew,
			* actOptionsJoin,
			* actOptionsScatter,
			* actOptionsOpenImage,
			* actOptionsFullscreen,
			* actOptionsPrefs,
			* actOptionsQuit
			;
	QMenu
			* menuOptions,
			* menuJigsaw,
			* menuHelp
			;

	QActionGroup	* menuGroupJigsaw;

	jigsawCanvas	* canvasWidget;

	void		createActions ();
	void		createMenus ();
};

