/*
	Copyright (C) 2013-2014 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include <QtGui>

// tableWidget->horizontalHeader ()->setClickable
#define QEX_SETCLICKABLE	setClickable

// tableWidget->horizontalHeader()->setResizeMode
#define QEX_RESIZEMODE		setResizeMode



#ifdef __cplusplus
	extern "C" {
#endif	//  __cplusplus


// C API

#include "mtkit.h"
#include "mtpixel.h"



#ifdef __cplusplus
	}
#endif	//  __cplusplus



QT_BEGIN_NAMESPACE

class	QLineEdit;
class	QPushButton;
class	QSignalMapper;
class	QString;
class	QTableWidget;
class	QTextEdit;
class	QWidget;

QT_END_NAMESPACE



class	qexArrowFilter;
class	qexButtonMenu;
class	qexImage;
class	qexImageArea;
class	qexPrefsWindow;



class qexPrefsWindow : public QDialog
{
	Q_OBJECT

public:
	qexPrefsWindow	( mtPrefs * prefsMem, QString title );
	~qexPrefsWindow	();

private slots:
	void		pressButtonFilter ();
	void		pressButtonReset ();
	void		pressButtonEdit ();
	void		pressButtonClose ();
	void		tableCellActivated ( int row, int column );
	void		tableCellChanged (
				int currentRow,
				int currentColumn,
				int previousRow,
				int previousColumn
				);

private:
	QLineEdit	* filterEdit,
			* infoEdit
			;
	QTableWidget	* tableWidget;
	QPushButton	* buttonReset,
			* buttonEdit;

	mtPrefs		* prefs;

	void		populateTable ();
};

/*
Allow arrow keys to move focus properly inside a grid layout of buttons.
Use:
	qexArrowFilter * arrowKeyFilter = new qexArrowFilter ( grid );
...
	button->installEventFilter ( arrowKeyFilter );
*/
class qexArrowFilter : public QObject
{
	Q_OBJECT

public:
	qexArrowFilter	( QGridLayout * layout );

protected:
	bool		eventFilter ( QObject * obj, QEvent * event);

private:
	QGridLayout	* gridLayout;
};

class qexImage : public QScrollArea
{
	Q_OBJECT

public:
	qexImage ();
	~qexImage ();

	mtImage		* getImage ();
	int		getZoom ();
	void		setImage ( mtImage * im );
	int		setZoom ( int z );
	void		update ();

private:
	qexImageArea	* area;
	mtImage		* image;
	int		zoom;
};

class qexImageArea : public QWidget
{
	Q_OBJECT

public:
	qexImageArea ( qexImage * parent );

private:
	void		paintEvent ( QPaintEvent * event );


	qexImage	* qi;
};

class qexButtonMenu : public QPushButton
{
	Q_OBJECT

public:
	qexButtonMenu ();
	~qexButtonMenu ();

	void		addItem ( QString t );
	void		clear ();
	int		count ();
	int		currentIndex ();
	int		findText ( QString t );
	void		setCurrentIndex ( int i );

signals:
	void		currentIndexChanged ( int i );

protected:
	void		keyPressEvent ( QKeyEvent * event );

public slots:
	void		popup ();

private slots:
	void		optionClicked ( int i );

private:
	QMenu		* menu;
	QSignalMapper	* signalMapper;
	int		itemCurrent;
};



namespace mtQEX
{
	QString qstringFromC (
		const char * cstring,	// If not NUL terminated cstring, set
		int size = -1		// size to length in bytes.
		);

	int fpickInitPrefs (		// Initialize default prefs for file
					// picker window
		mtPrefs * prefs,	// Call this before mtkit_prefs_load
		const char * prefs_prefix
		);

	int cpickInitPrefs (		// Initialize default prefs for colour
					// picker window
		mtPrefs * prefs,	// Call this before mtkit_prefs_load
		const char * prefs_prefix
		);

	int prefsInitPrefs (		// Initialize default prefs for
					// preferences window
		mtPrefs * prefs		// Call this before mtkit_prefs_load
		);

	int prefsWindowMirrorPrefs (
		mtPrefs		* dest,
		mtPrefs		* src
		);

	QString dialogTextLine (	// Single line text entry
		QWidget * parent,
		QString title,
		QString label,
		QString text,		// Initial text to populate dialog
		int maxLength = -1,	// Maximum number of characters -1 = any
		QLineEdit::EchoMode mode = QLineEdit :: Normal
		);

	QString dialogText (		// Multiple line text entry
		QWidget * parent,
		QString title,
		QString label,
		QString text		// Initial text to populate dialog
		);
}






/*

// FIXME - When mtGEX dies can we deprecate this as it isn't used???

These entries are required in the prefs table (prefix before each key) if you
don't use fpickInitPrefs:
(if you don't you will forever live with defaults!)

{ "fpick_case_insensitive", MTKIT_PREF_TYPE_INT, "1" },
{ "fpick_show_hidden",	MTKIT_PREF_TYPE_INT, "0" },

{ "fpick_window_x",	MTKIT_PREF_TYPE_INT, "0" },
{ "fpick_window_y",	MTKIT_PREF_TYPE_INT, "0" },
{ "fpick_window_w",	MTKIT_PREF_TYPE_INT, "600" },
{ "fpick_window_h",	MTKIT_PREF_TYPE_INT, "500" },

{ "fpick_dir_1",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_2",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_3",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_4",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_5",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_6",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_7",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_8",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_9",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_10",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_11",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_12",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_13",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_14",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_15",	MTKIT_PREF_TYPE_DIR },
{ "fpick_dir_16",	MTKIT_PREF_TYPE_DIR },

{ "fpick_col1",		MTKIT_PREF_TYPE_INT, "250" },
{ "fpick_col2",		MTKIT_PREF_TYPE_INT, "64" },
{ "fpick_col3",		MTKIT_PREF_TYPE_INT, "80" },
{ "fpick_col4",		MTKIT_PREF_TYPE_INT, "150" },



// FIXME - When mtGEX dies can we deprecate this as it isn't used???

These entries are required in the prefs table (prefix before each key) if you
don't use cpickInitPrefs:
(if you don't you will forever live with defaults!)

{ "cpick_pal_1",	MTKIT_PREF_TYPE_RGB, "0" },
{ "cpick_pal_2",	MTKIT_PREF_TYPE_RGB, "16711680" },
{ "cpick_pal_3",	MTKIT_PREF_TYPE_RGB, "16776960" },
{ "cpick_pal_4",	MTKIT_PREF_TYPE_RGB, "65280" },
{ "cpick_pal_5",	MTKIT_PREF_TYPE_RGB, "255" },
{ "cpick_pal_6",	MTKIT_PREF_TYPE_RGB, "16711935" },
{ "cpick_pal_7",	MTKIT_PREF_TYPE_RGB, "65535" },
{ "cpick_pal_8",	MTKIT_PREF_TYPE_RGB, "16777215" },
{ "cpick_pal_9",	MTKIT_PREF_TYPE_RGB, "14408667" },
{ "cpick_pal_10",	MTKIT_PREF_TYPE_RGB, "11974326" },
{ "cpick_pal_11",	MTKIT_PREF_TYPE_RGB, "9605778" },
{ "cpick_pal_12",	MTKIT_PREF_TYPE_RGB, "7171437" },
{ "cpick_pal_13",	MTKIT_PREF_TYPE_RGB, "4802889" },
{ "cpick_pal_14",	MTKIT_PREF_TYPE_RGB, "2368548" },
{ "cpick_pal_15",	MTKIT_PREF_TYPE_RGB, "14389760" },
{ "cpick_pal_16",	MTKIT_PREF_TYPE_RGB, "7190089" },



// FIXME - When mtGEX dies can we deprecate this as it isn't used???

You must set these up in the static lists somewhere if you don't use
prefsInitPrefs:
(if you don't you will forever live with defaults!)
NOTE: These do not appear in the editable list.

{ "prefs.col1",		MTKIT_PREF_TYPE_INT, "150" },
{ "prefs.col2",		MTKIT_PREF_TYPE_INT, "50" },
{ "prefs.col3",		MTKIT_PREF_TYPE_INT, "50" },
{ "prefs.col4",		MTKIT_PREF_TYPE_INT, "50" },

{ "prefs.window_x",	MTKIT_PREF_TYPE_INT, "259" },
{ "prefs.window_y",	MTKIT_PREF_TYPE_INT, "259" },
{ "prefs.window_w",	MTKIT_PREF_TYPE_INT, "693" },
{ "prefs.window_h",	MTKIT_PREF_TYPE_INT, "651" },

{ "prefs.file.window_x",	MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.file.window_y",	MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.file.window_w",	MTKIT_PREF_TYPE_INT, "600" },
{ "prefs.file.window_h",	MTKIT_PREF_TYPE_INT, "500" },

{ "prefs.color.window_x",	MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.color.window_y",	MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.color.window_w",	MTKIT_PREF_TYPE_INT, "100" },
{ "prefs.color.window_h",	MTKIT_PREF_TYPE_INT, "100" },

{ "prefs.edit.window_x",	MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.edit.window_y",	MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.edit.window_w",	MTKIT_PREF_TYPE_INT, "400" },
{ "prefs.edit.window_h",	MTKIT_PREF_TYPE_INT, "100" },

{ "prefs.cpick_pal_1",	MTKIT_PREF_TYPE_RGB, "0" },
{ "prefs.cpick_pal_2",	MTKIT_PREF_TYPE_RGB, "16711680" },
{ "prefs.cpick_pal_3",	MTKIT_PREF_TYPE_RGB, "16776960" },
{ "prefs.cpick_pal_4",	MTKIT_PREF_TYPE_RGB, "65280" },
{ "prefs.cpick_pal_5",	MTKIT_PREF_TYPE_RGB, "255" },
{ "prefs.cpick_pal_6",	MTKIT_PREF_TYPE_RGB, "16711935" },
{ "prefs.cpick_pal_7",	MTKIT_PREF_TYPE_RGB, "65535" },
{ "prefs.cpick_pal_8",	MTKIT_PREF_TYPE_RGB, "16777215" },
{ "prefs.cpick_pal_9",	MTKIT_PREF_TYPE_RGB, "14408667" },
{ "prefs.cpick_pal_10",	MTKIT_PREF_TYPE_RGB, "11974326" },
{ "prefs.cpick_pal_11",	MTKIT_PREF_TYPE_RGB, "9605778" },
{ "prefs.cpick_pal_12",	MTKIT_PREF_TYPE_RGB, "7171437" },
{ "prefs.cpick_pal_13",	MTKIT_PREF_TYPE_RGB, "4802889" },
{ "prefs.cpick_pal_14",	MTKIT_PREF_TYPE_RGB, "2368548" },
{ "prefs.cpick_pal_15",	MTKIT_PREF_TYPE_RGB, "14389760" },
{ "prefs.cpick_pal_16",	MTKIT_PREF_TYPE_RGB, "7190089" },

{ "prefs.fpick_dir_1",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_2",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_3",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_4",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_5",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_6",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_7",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_8",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_9",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_10",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_11",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_12",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_13",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_14",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_15",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_16",	MTKIT_PREF_TYPE_DIR },

{ "prefs.fpick_case_insensitive", MTKIT_PREF_TYPE_INT, "1" },
{ "prefs.fpick_show_hidden", MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.fpick_col1",	MTKIT_PREF_TYPE_INT, "250" },
{ "prefs.fpick_col2",	MTKIT_PREF_TYPE_INT, "64" },
{ "prefs.fpick_col3",	MTKIT_PREF_TYPE_INT, "80" },
{ "prefs.fpick_col4",	MTKIT_PREF_TYPE_INT, "150" },

*/
