/*
	Copyright (C) 2013 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "private.h"



QString mtQEX :: qstringFromC (
	const char	* cstring,
	int		size
	)
{
	QString		result;


	if ( size < 0 )
	{
		size = -1;
	}

	if ( ! cstring )
	{
		result = "";
	}
	else if ( mtkit_utf8_string_legal ( (unsigned char *)cstring, size ) )
	{
		result = QString :: fromUtf8 ( cstring, size );
	}
	else
	{
		result = QString :: fromLatin1 ( cstring, size );
	}

	return result;
}

int mtQEX :: fpickInitPrefs (
	mtPrefs		* prefs,
	const char	* prefs_prefix
	)
{
	mtPrefItem prefs_table[] = {
		{ "fpick_case_insensitive", MTKIT_PREF_TYPE_INT, "1" },
		{ "fpick_show_hidden",	MTKIT_PREF_TYPE_INT, "0" },

		{ "fpick_window_x",	MTKIT_PREF_TYPE_INT, "50" },
		{ "fpick_window_y",	MTKIT_PREF_TYPE_INT, "50" },
		{ "fpick_window_w",	MTKIT_PREF_TYPE_INT, "500" },
		{ "fpick_window_h",	MTKIT_PREF_TYPE_INT, "500" },

		{ "fpick_dir_1",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_2",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_3",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_4",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_5",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_6",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_7",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_8",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_9",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_10",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_11",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_12",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_13",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_14",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_15",	MTKIT_PREF_TYPE_DIR },
		{ "fpick_dir_16",	MTKIT_PREF_TYPE_DIR },

		{ "fpick_col1",		MTKIT_PREF_TYPE_INT, "250" },
		{ "fpick_col2",		MTKIT_PREF_TYPE_INT, "64" },
		{ "fpick_col3",		MTKIT_PREF_TYPE_INT, "80" },
		{ "fpick_col4",		MTKIT_PREF_TYPE_INT, "150" },

		{ NULL }
		};


	return mtkit_prefs_add ( prefs, prefs_table, prefs_prefix );
}

int mtQEX :: cpickInitPrefs (
	mtPrefs		* prefs,
	const char	* prefs_prefix
	)
{
	mtPrefItem prefs_table[] = {
		{ "cpick_pal_1",	MTKIT_PREF_TYPE_RGB, "0" },
		{ "cpick_pal_2",	MTKIT_PREF_TYPE_RGB, "16711680" },
		{ "cpick_pal_3",	MTKIT_PREF_TYPE_RGB, "16776960" },
		{ "cpick_pal_4",	MTKIT_PREF_TYPE_RGB, "65280" },
		{ "cpick_pal_5",	MTKIT_PREF_TYPE_RGB, "255" },
		{ "cpick_pal_6",	MTKIT_PREF_TYPE_RGB, "16711935" },
		{ "cpick_pal_7",	MTKIT_PREF_TYPE_RGB, "65535" },
		{ "cpick_pal_8",	MTKIT_PREF_TYPE_RGB, "16777215" },
		{ "cpick_pal_9",	MTKIT_PREF_TYPE_RGB, "14408667" },
		{ "cpick_pal_10",	MTKIT_PREF_TYPE_RGB, "11974326" },
		{ "cpick_pal_11",	MTKIT_PREF_TYPE_RGB, "9605778" },
		{ "cpick_pal_12",	MTKIT_PREF_TYPE_RGB, "7171437" },
		{ "cpick_pal_13",	MTKIT_PREF_TYPE_RGB, "4802889" },
		{ "cpick_pal_14",	MTKIT_PREF_TYPE_RGB, "2368548" },
		{ "cpick_pal_15",	MTKIT_PREF_TYPE_RGB, "14389760" },
		{ "cpick_pal_16",	MTKIT_PREF_TYPE_RGB, "7190089" },

		{ NULL }
		};


	return mtkit_prefs_add ( prefs, prefs_table, prefs_prefix );
}



static mtPrefItem default_prefs_table[] = {
{ "prefs.col1",		MTKIT_PREF_TYPE_INT, "150" },
{ "prefs.col2",		MTKIT_PREF_TYPE_INT, "70" },
{ "prefs.col3",		MTKIT_PREF_TYPE_INT, "70" },
{ "prefs.col4",		MTKIT_PREF_TYPE_INT, "50" },

{ "prefs.window_x",	MTKIT_PREF_TYPE_INT, "50" },
{ "prefs.window_y",	MTKIT_PREF_TYPE_INT, "50" },
{ "prefs.window_w",	MTKIT_PREF_TYPE_INT, "500" },
{ "prefs.window_h",	MTKIT_PREF_TYPE_INT, "500" },

{ "prefs.file.window_x",	MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.file.window_y",	MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.file.window_w",	MTKIT_PREF_TYPE_INT, "600" },
{ "prefs.file.window_h",	MTKIT_PREF_TYPE_INT, "500" },

{ "prefs.color.window_x",	MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.color.window_y",	MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.color.window_w",	MTKIT_PREF_TYPE_INT, "100" },
{ "prefs.color.window_h",	MTKIT_PREF_TYPE_INT, "100" },

{ "prefs.edit.window_x",	MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.edit.window_y",	MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.edit.window_w",	MTKIT_PREF_TYPE_INT, "400" },
{ "prefs.edit.window_h",	MTKIT_PREF_TYPE_INT, "100" },

{ "prefs.cpick_pal_1",	MTKIT_PREF_TYPE_RGB, "0" },
{ "prefs.cpick_pal_2",	MTKIT_PREF_TYPE_RGB, "16711680" },
{ "prefs.cpick_pal_3",	MTKIT_PREF_TYPE_RGB, "16776960" },
{ "prefs.cpick_pal_4",	MTKIT_PREF_TYPE_RGB, "65280" },
{ "prefs.cpick_pal_5",	MTKIT_PREF_TYPE_RGB, "255" },
{ "prefs.cpick_pal_6",	MTKIT_PREF_TYPE_RGB, "16711935" },
{ "prefs.cpick_pal_7",	MTKIT_PREF_TYPE_RGB, "65535" },
{ "prefs.cpick_pal_8",	MTKIT_PREF_TYPE_RGB, "16777215" },
{ "prefs.cpick_pal_9",	MTKIT_PREF_TYPE_RGB, "14408667" },
{ "prefs.cpick_pal_10",	MTKIT_PREF_TYPE_RGB, "11974326" },
{ "prefs.cpick_pal_11",	MTKIT_PREF_TYPE_RGB, "9605778" },
{ "prefs.cpick_pal_12",	MTKIT_PREF_TYPE_RGB, "7171437" },
{ "prefs.cpick_pal_13",	MTKIT_PREF_TYPE_RGB, "4802889" },
{ "prefs.cpick_pal_14",	MTKIT_PREF_TYPE_RGB, "2368548" },
{ "prefs.cpick_pal_15",	MTKIT_PREF_TYPE_RGB, "14389760" },
{ "prefs.cpick_pal_16",	MTKIT_PREF_TYPE_RGB, "7190089" },

{ "prefs.fpick_dir_1",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_2",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_3",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_4",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_5",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_6",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_7",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_8",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_9",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_10",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_11",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_12",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_13",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_14",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_15",	MTKIT_PREF_TYPE_DIR },
{ "prefs.fpick_dir_16",	MTKIT_PREF_TYPE_DIR },

{ "prefs.fpick_case_insensitive", MTKIT_PREF_TYPE_INT, "1" },
{ "prefs.fpick_show_hidden", MTKIT_PREF_TYPE_INT, "0" },
{ "prefs.fpick_col1",	MTKIT_PREF_TYPE_INT, "250" },
{ "prefs.fpick_col2",	MTKIT_PREF_TYPE_INT, "64" },
{ "prefs.fpick_col3",	MTKIT_PREF_TYPE_INT, "80" },
{ "prefs.fpick_col4",	MTKIT_PREF_TYPE_INT, "150" },

{ NULL }

	};


int mtQEX :: prefsInitPrefs (
	mtPrefs		* prefs
	)
{
	return mtkit_prefs_add ( prefs, default_prefs_table, 0 );
}

int mtQEX :: prefsWindowMirrorPrefs (
	mtPrefs		* dest,
	mtPrefs		* src
	)
{
	return mtkit_prefs_value_mirror ( dest, src, default_prefs_table );
}

qexArrowFilter :: qexArrowFilter ( QGridLayout * layout )
	:
	gridLayout	( layout )
{
}

bool qexArrowFilter :: eventFilter ( QObject * obj, QEvent * event )
{
	QKeyEvent	* keyEvent = NULL;
	QWidget		* widget = NULL;
	int		idx = -1;


	if ( event->type () == QEvent::KeyPress )
	{
		keyEvent = static_cast<QKeyEvent *>(event);
		widget = qobject_cast<QWidget *>(obj);
		idx = gridLayout->indexOf ( widget );
	}

	if ( idx >= 0 )
	{
		int		dr = 0,
				dc = 0,
				r = 0,
				c = 0,
				j;


		switch ( keyEvent->key () )
		{
		case Qt::Key_Up:	dr = -1;	break;
		case Qt::Key_Down:	dr = 1;		break;
		case Qt::Key_Left:	dc = -1;	break;
		case Qt::Key_Right:	dc = 1;		break;
		}

		if ( dr || dc )
		{
			QLayoutItem	* item;


			gridLayout->getItemPosition ( idx, &r, &c, &j, &j );

			r += dr;
			c += dc;

			item = gridLayout->itemAtPosition ( r, c );

			if ( item && item->widget () )
			{
				item->widget ()->setFocus (
					Qt::OtherFocusReason );
			}

			return true;
		}
	}

	// standard event processing
	return QObject :: eventFilter ( obj, event );
}

