/*
	Copyright (C) 2013-2014 Mark Tyler

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program in the file COPYING.
*/

#include "private.h"



enum
{
	COLUMN_KEY		= 0,
	COLUMN_STATUS		= 1,
	COLUMN_TYPE		= 2,
	COLUMN_VALUE		= 3,

	COLUMN_TOTAL		= 4
};



qexPrefsWindow :: qexPrefsWindow (
	mtPrefs		* prefsMem,
	QString		title
	)
	:
	prefs		( prefsMem )
{
	int		wx = 50,
			wy = 50,
			ww = 500,
			wh = 500,
			i,
			cw
			;
	char		txt[ 256 ];
	QVBoxLayout	* vlayout;
	QHBoxLayout	* hRow;
	QPushButton	* button;
	QStringList	columnLabels;


	setWindowModality ( Qt::ApplicationModal );

	setWindowTitle ( title );

	mtkit_prefs_get_int ( prefs, "prefs.window_x", &wx );
	mtkit_prefs_get_int ( prefs, "prefs.window_y", &wy );
	mtkit_prefs_get_int ( prefs, "prefs.window_w", &ww );
	mtkit_prefs_get_int ( prefs, "prefs.window_h", &wh );

	setGeometry ( wx, wy, ww, wh );

	vlayout = new QVBoxLayout;
	this->setLayout ( vlayout );

	hRow = new QHBoxLayout ();
	vlayout->addLayout ( hRow );

	button = new QPushButton ( tr ( "&Filter" ) );
	button->setAutoDefault ( false );
	connect ( button, SIGNAL ( clicked () ), this,
		SLOT ( pressButtonFilter () ) );
	hRow->addWidget ( button );

	filterEdit = new QLineEdit ();
	hRow->addWidget ( filterEdit );
	connect ( filterEdit, SIGNAL ( returnPressed () ), this,
		SLOT ( pressButtonFilter () ) );

	tableWidget = new QTableWidget;
	vlayout->addWidget ( tableWidget );

	tableWidget->setSelectionMode ( QAbstractItemView::SingleSelection );
	tableWidget->setSelectionBehavior ( QAbstractItemView::SelectRows );
	tableWidget->setEditTriggers ( QAbstractItemView::NoEditTriggers );
	tableWidget->setColumnCount ( 4 );
	tableWidget->setShowGrid ( false );

	tableWidget->verticalHeader ()->hide ();
	tableWidget->verticalHeader ()->setDefaultSectionSize (
		tableWidget->verticalHeader ()->fontMetrics ().height () + 4 );

	tableWidget->horizontalHeader ()->QEX_SETCLICKABLE ( false );
	tableWidget->horizontalHeader ()->setStretchLastSection ( true );

	columnLabels
		<< tr ( "Key" )
		<< tr ( "Status" )
		<< tr ( "Type" )
		<< tr ( "Value" )
		;
	tableWidget->setHorizontalHeaderLabels ( columnLabels );

	// Set column widths
	for ( i = 0; i < COLUMN_TOTAL - 1; i++ )
	{
		sprintf ( txt, "prefs.col%i", i + 1 );

		if ( mtkit_prefs_get_int ( prefs, txt, &cw ) )
		{
			continue;
		}

		tableWidget->horizontalHeader ()-> resizeSection ( i, cw );
	}

	connect ( tableWidget, SIGNAL ( cellActivated ( int, int ) ),
		this, SLOT ( tableCellActivated ( int, int ) ) );

	connect ( tableWidget, SIGNAL( currentCellChanged (int, int, int, int)),
		this, SLOT ( tableCellChanged ( int, int, int, int ) ) );

	hRow = new QHBoxLayout ();
	vlayout->addLayout ( hRow );

	infoEdit = new QLineEdit ();
	hRow->addWidget ( infoEdit );
	infoEdit->setReadOnly ( true );

	button = new QPushButton ( QIcon :: fromTheme ( "edit-clear" ),
		tr ( "&Reset" ) );
	button->setAutoDefault ( false );
	connect ( button, SIGNAL ( clicked () ), this,
		SLOT ( pressButtonReset () ) );
	hRow->addWidget ( button );
	buttonReset = button;

	button = new QPushButton ( QIcon :: fromTheme ( "document-properties" ),
		tr ( "&Edit" ) );
	button->setAutoDefault ( false );
	connect ( button, SIGNAL ( clicked () ), this,
		SLOT ( pressButtonEdit () ) );
	hRow->addWidget ( button );
	buttonEdit = button;

	button = new QPushButton ( QIcon :: fromTheme ( "window-close" ),
		tr ( "&Close" ) );
	button->setAutoDefault ( false );
	connect ( button, SIGNAL ( clicked () ), this,
		SLOT ( pressButtonClose () ) );
	hRow->addWidget ( button );

	filterEdit->setFocus ();
	show ();
	populateTable ();

	exec ();
}

qexPrefsWindow :: ~qexPrefsWindow ()
{
	int		i,
			cw;
	char		txt[ 256 ];


	mtkit_prefs_set_int ( prefs, "prefs.window_x", geometry().x () );
	mtkit_prefs_set_int ( prefs, "prefs.window_y", geometry().y () );
	mtkit_prefs_set_int ( prefs, "prefs.window_w", geometry().width () );
	mtkit_prefs_set_int ( prefs, "prefs.window_h", geometry().height () );

	// Set column widths
	for ( i = 0; i < COLUMN_TOTAL - 1; i++ )
	{
		sprintf ( txt, "prefs.col%i", i + 1 );

		cw = tableWidget->horizontalHeader ()-> sectionSize ( i );
		mtkit_prefs_set_int ( prefs, txt, cw );
	}
}

void qexPrefsWindow :: pressButtonFilter ()
{
	populateTable ();
}

static mtPrefItemValue * get_piv (
	QTableWidget	* tableWidget,
	int		row
	)
{
	QTableWidgetItem * twItem;


	twItem = tableWidget->item ( row, COLUMN_KEY );

	if ( ! twItem )
	{
		return NULL;
	}

	return (mtPrefItemValue *)twItem->data( Qt::UserRole ).value<void *> ();
}

static void update_table_status_value (
	QTableWidget	* tableWidget,
	mtPrefItemValue	* piv,
	int		row
	)
{
	const char	* st;
	char		buf[ 256 ] =  { 0 },
			* bp;
	QTableWidgetItem * twItem;


	if ( piv->pref_item->def )
	{
		if ( strcmp ( piv->value, piv->pref_item->def ) == 0 )
		{
			st = "default";
		}
		else
		{
			st = "user set";
		}
	}
	else		// Default is NULL
	{
		if ( piv->value[0] == 0 )
		{
			st = "default";
		}
		else
		{
			st = "user set";
		}
	}

	twItem = new QTableWidgetItem;
	twItem->setText ( mtQEX :: qstringFromC ( (char *)st ) );
	tableWidget->setItem ( row, COLUMN_STATUS, twItem );

	mtkit_prefs_get_str_val ( piv, piv->value, buf, sizeof ( buf ) );

	// Remove any newlines
	while ( ( bp = strchr ( buf, '\n' ) ) )
	{
		bp[0] = ' ';
	}

	twItem = new QTableWidgetItem;
	twItem->setText ( mtQEX :: qstringFromC ( buf ) );
	tableWidget->setItem ( row, COLUMN_VALUE, twItem );
}

void qexPrefsWindow :: pressButtonReset ()
{
	int		row;
	mtPrefItemValue	* piv;


	row = tableWidget->currentRow ();
	piv = get_piv ( tableWidget, row );

	if ( ! piv )
	{
		return;
	}

	mtkit_prefs_set_default ( prefs, piv->pref_item->key );
	update_table_status_value ( tableWidget, piv, row );
}

static int winget_int (
	qexPrefsWindow	* win,
	mtPrefs		* prefs,
	mtPrefItemValue	* piv
	)
{
	bool		ok;
	int		num,
			val = 0
			;
	double		min = -1000000,
			max = 1000000
			;


	if ( piv->pref_item->opt )
	{
		mtkit_strtok_num ( piv->pref_item->opt, "\t", 0, &min );
		mtkit_strtok_num ( piv->pref_item->opt, "\t", 1, &max );
	}

	mtkit_prefs_get_int ( prefs, piv->pref_item->key, &val );

	num = QInputDialog :: getInt ( win, "Edit Integer",
		piv->pref_item->key, val, min, max, 1, &ok );

	if ( ok )
	{
		mtkit_prefs_set_int ( prefs, piv->pref_item->key, num );

		return 1;		// Changed
	}

	return 0;			// No change
}

static int winget_bool (
	qexPrefsWindow	* win,
	mtPrefs		* prefs,
	mtPrefItemValue	* piv
	)
{
	bool		ok;
	int		val = 0;
	QString		res;
	QStringList	items;


	mtkit_prefs_get_int ( prefs, piv->pref_item->key, &val );

	items << "False" << "True";

	res = QInputDialog :: getItem ( win, "Edit Boolean",
		piv->pref_item->key, items, val, false, &ok );

	if ( ok && ! res.isEmpty () )
	{
		int		num;


		num = items.indexOf ( res, 0 );
		mtkit_prefs_set_int ( prefs, piv->pref_item->key, num );

		return 1;		// Changed
	}

	return 0;			// No change
}

static int winget_option (
	qexPrefsWindow	* win,
	mtPrefs		* prefs,
	mtPrefItemValue	* piv
	)
{
	bool		ok;
	int		val = 0,
			i;
	char		* txt;
	QString		res;
	QStringList	items;


	if ( ! piv->pref_item->opt )
	{
		return 0;		// No change
	}

	mtkit_prefs_get_int ( prefs, piv->pref_item->key, &val );

	for ( i = 0; ; i++ )
	{
		txt = mtkit_strtok ( piv->pref_item->opt, "\t", i );
		if ( ! txt )
		{
			break;		// End of list
		}

		items << txt;

		free ( txt );
	}

	res = QInputDialog :: getItem ( win, "Edit Option",
		piv->pref_item->key, items, val, false, &ok );

	if ( ok && ! res.isEmpty () )
	{
		int		num;


		num = items.indexOf ( res, 0 );
		mtkit_prefs_set_int ( prefs, piv->pref_item->key, num );

		return 1;		// Changed
	}


	return 0;			// No change
}

static int winget_double (
	qexPrefsWindow	* win,
	mtPrefs		* prefs,
	mtPrefItemValue	* piv
	)
{
	bool		ok;
	double		min = -1000000,
			max = 1000000,
			val = 0,
			dp = 3,
			num
			;


	if ( piv->pref_item->opt )
	{
		double		a = 0,
				b = 0,
				c = 0;

		mtkit_strtok_num ( piv->pref_item->opt, "\t", 0, &a );
		mtkit_strtok_num ( piv->pref_item->opt, "\t", 1, &b );
		mtkit_strtok_num ( piv->pref_item->opt, "\t", 2, &c );

		if ( b > a )
		{
			// User is setting min/max in first 2 args
			min = a;
			max = b;

			if ( c > 0 && c < 15 )
			{
				// Use is setting decimal places in 3rd arg
				dp = c;
			}
		}
		else
		{
			if ( a > 0 && a < 15 )
			{
				// Use is setting decimal places in 1st arg
				dp = a;
			}
		}
// FIXME - when mtGEX dies, leave a note in mtkit.h regarding 1st/3rd arg for doubles
	}

	mtkit_prefs_get_double ( prefs, piv->pref_item->key, &val );

	num = QInputDialog :: getDouble ( win, "Edit Decimal",
		piv->pref_item->key, val, min, max, dp, &ok );

	if ( ok )
	{
		mtkit_prefs_set_double ( prefs, piv->pref_item->key, num );

		return 1;		// Changed
	}

	return 0;			// No change
}

static int winget_string (
	qexPrefsWindow	* win,
	mtPrefs		* prefs,
	mtPrefItemValue	* piv
	)
{
	double		maxLen = -1;


	if ( piv->pref_item->opt )
	{
		mtkit_strtok_num ( piv->pref_item->opt, "\t", 0, &maxLen );
	}

	QString text = mtQEX :: dialogTextLine ( win, "Edit Text",
		piv->pref_item->key, piv->value, maxLen );

	if ( ! text.isEmpty () )
	{
		mtkit_prefs_set_str ( prefs, piv->pref_item->key,
			text.toUtf8 ().data () );

		return 1;		// Changed
	}

	return 0;			// No change
}

static int winget_string_multi (
	qexPrefsWindow	* win,
	mtPrefs		* prefs,
	mtPrefItemValue	* piv
	)
{
	QString text = mtQEX :: dialogText ( win, "Edit Text",
		piv->pref_item->key, piv->value );


	if ( ! text.isEmpty () )
	{
		mtkit_prefs_set_str ( prefs, piv->pref_item->key,
			text.toUtf8 ().data () );

		return 1;		// Changed
	}

	return 0;			// No change
}

static int winget_rgb (
	qexPrefsWindow	* win,
	mtPrefs		* prefs,
	mtPrefItemValue	* piv
	)
{
	int		colRGB = 0;
	QColor		color;


	mtkit_prefs_get_int ( prefs, piv->pref_item->key, &colRGB );

	color = QColorDialog :: getColor ( QColor ( MTPIXEL_INT_2_R ( colRGB ),
			MTPIXEL_INT_2_G ( colRGB ),
			MTPIXEL_INT_2_B ( colRGB )  ),
		win, piv->pref_item->key, QColorDialog :: DontUseNativeDialog );

	if ( color.isValid () )
	{
		colRGB = MTPIXEL_RGB_2_INT ( color.red (), color.green (),
			color.blue () );

		mtkit_prefs_set_int ( prefs, piv->pref_item->key,
			colRGB );

		return 1;		// Changed
	}

	return 0;			// No change
}

static int winget_directory (
	qexPrefsWindow	* win,
	mtPrefs		* prefs,
	mtPrefItemValue	* piv
	)
{
	QString filename = QFileDialog :: getExistingDirectory ( win,
		piv->pref_item->key, piv->value,
		QFileDialog::DontUseNativeDialog | QFileDialog::ShowDirsOnly );


	if ( ! filename.isEmpty () )
	{
		mtkit_prefs_set_str ( prefs, piv->pref_item->key,
			filename.toUtf8 ().data () );

		return 1;		// Changed
	}

	return 0;			// No change
}

static int winget_file (
	qexPrefsWindow	* win,
	mtPrefs		* prefs,
	mtPrefItemValue	* piv
	)
{
	QString filename = QFileDialog :: getOpenFileName ( win,
		piv->pref_item->key, piv->value, NULL,
		NULL, QFileDialog :: DontUseNativeDialog );


	if ( ! filename.isEmpty () )
	{
		mtkit_prefs_set_str ( prefs, piv->pref_item->key,
			filename.toUtf8 ().data () );

		return 1;		// Changed
	}

	return 0;			// No change
}

void qexPrefsWindow :: pressButtonEdit ()
{
	int		row,
			change = 0;
	mtPrefItemValue	* piv;


	row = tableWidget->currentRow ();
	piv = get_piv ( tableWidget, row );

	if ( ! piv )
	{
		return;
	}

	switch ( piv->pref_item->type )
	{
	case MTKIT_PREF_TYPE_INT:
		change = winget_int ( this, prefs, piv );
		break;

	case MTKIT_PREF_TYPE_BOOL:
		change = winget_bool ( this, prefs, piv );
		break;

	case MTKIT_PREF_TYPE_OPTION:
		change = winget_option ( this, prefs, piv );
		break;

	case MTKIT_PREF_TYPE_DOUBLE:
		change = winget_double ( this, prefs, piv );
		break;

	case MTKIT_PREF_TYPE_STR:
		change = winget_string ( this, prefs, piv );
		break;

	case MTKIT_PREF_TYPE_STR_MULTI:
		change = winget_string_multi ( this, prefs, piv );
		break;

	case MTKIT_PREF_TYPE_RGB:
		change = winget_rgb ( this, prefs, piv );
		break;

	case MTKIT_PREF_TYPE_DIR:
		change = winget_directory ( this, prefs, piv );
		break;

	case MTKIT_PREF_TYPE_FILE:
		change = winget_file ( this, prefs, piv );
		break;

	default:
		break;
	}

	if ( change )
	{
		update_table_status_value ( tableWidget, piv, row );
	}
}

void qexPrefsWindow :: pressButtonClose ()
{
	close ();
}

static void populate_recurse (
	QTableWidget	* tableWidget,
	mtTreeNode	* node,
	char		* filter
	)
{
	mtPrefItemValue	* piv;


	if ( ! node )
	{
		return;
	}

	populate_recurse ( tableWidget, node->left, filter );
	piv = (mtPrefItemValue *)node->data;

	if (	piv &&
		strncmp ( piv->pref_item->key, "prefs.", 6 ) &&
		( ! filter || mtkit_strcasestr ( piv->pref_item->key, filter ) )
		)
	{
		int		row;
		QTableWidgetItem * twItem;


		row = tableWidget->rowCount ();
		tableWidget->setRowCount ( row + 1 );

		twItem = new QTableWidgetItem;
		twItem->setText ( mtQEX :: qstringFromC (
			(char *)piv->pref_item->key ) );
		tableWidget->setItem ( row, COLUMN_KEY, twItem );

		twItem->setData ( Qt::UserRole,
			QVariant::fromValue ( (void *)piv ) );

		twItem = new QTableWidgetItem;
		twItem->setText ( mtQEX :: qstringFromC (
			mtkit_prefs_type_text ( piv->pref_item->type ) ) );
		tableWidget->setItem ( row, COLUMN_TYPE, twItem );

		update_table_status_value ( tableWidget, piv, row );
	}

	populate_recurse ( tableWidget, node->right, filter );
}

void qexPrefsWindow :: populateTable ()
{
	mtTree		* tree;


	tree = mtkit_prefs_get_tree ( prefs );

	if ( tree )
	{
		tableWidget->clearContents ();
		tableWidget->setRowCount ( 0 );

		populate_recurse ( tableWidget, tree->root,
			filterEdit->text ().toUtf8 ().data () );

		tableWidget->setCurrentCell ( 0, 0 );
	}

	if ( tableWidget->rowCount () < 1 )
	{
		buttonReset->setEnabled ( false );
		buttonEdit->setEnabled ( false );
	}
	else
	{
		buttonReset->setEnabled ( true );
		buttonEdit->setEnabled ( true );
	}
}

void qexPrefsWindow :: tableCellActivated (
	int		row,
	int		column
	)
{
	pressButtonEdit ();
}

void qexPrefsWindow :: tableCellChanged (
	int		currentRow,
	int		currentColumn,
	int		previousRow,
	int		previousColumn
	)
{
	mtPrefItemValue	* piv;


	piv = get_piv ( tableWidget, currentRow );

	if ( piv )
	{
		QString qs = mtQEX :: qstringFromC (
			(char *)piv->pref_item->description );


		infoEdit->setText ( qs );
		infoEdit->setToolTip ( qs );
	}
}

